# -*- coding: utf-8 -*-
import pytest
import time
import sys
import cPickle as pickle
from test_base_class import TestBaseClass
from aerospike import predicates as p

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestQueryApply(object):
    def setup_method(self, method):
        """
        Setup method.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(user, password)
        self.client.index_integer_create('test', 'demo', 'age',
                'test_demo_age_idx')
        self.client.index_integer_create('test', None, 'age',
                'test_null_age_idx')
        for i in xrange(1, 5):
            key = ('test', 'demo', i)
            rec = {'name': 'name%s' % (str(i)), 'age': i}
            self.client.put(key, rec)
        policy = {}
        self.client.udf_put(u"bin_lua.lua", 0, policy)

    def teardown_method(self, method):
        """
        Teardoen method.
        """
        for i in xrange(1, 5):
            key = ('test', 'demo', i)
            try:
                self.client.remove(key)
            except TimeoutError:
                pass
        self.client.close()

    def test_query_apply_with_no_parameters(self):
        """
        Invoke query_apply() without any mandatory parameters.
        """
        with pytest.raises(TypeError) as typeError:
            self.client.query_apply()
        assert "Required argument 'ns' (pos 1) not found" in typeError.value

    def test_query_apply_with_correct_parameters(self):
        """
        Invoke query_apply() with correct parameters
        """
        query_id = self.client.query_apply("test", "demo", p.between("age", 1,
            5), "bin_lua", "mytransform", ['age', 2])

        time.sleep(0.1)
        while True:
            response = self.client.job_info(query_id, aerospike.JOB_QUERY)
            if response['status'] != aerospike.JOB_STATUS_INPROGRESS:
                break
            time.sleep(0.1)

        for i in xrange(1, 5):
            key = ('test', 'demo', i)
            (key, meta, bins) = self.client.get(key)
            if bins['age'] != i + 2:
                assert True == False

        assert True == True

    def test_query_apply_with_correct_policy(self):
        """
        Invoke query_apply() with correct policy
        """
        policy = {'timeout': 1000}
        query_id = self.client.query_apply("test", "demo", p.between("age", 1,
            5), "bin_lua", "mytransform", ['age', 2], policy)

        time.sleep(0.1)
        while True:
            response = self.client.job_info(query_id, aerospike.JOB_QUERY)
            if response['status'] != aerospike.JOB_STATUS_INPROGRESS:
                break
            time.sleep(0.1)

        for i in xrange(1, 5):
            key = ('test', 'demo', i)
            (key, meta, bins) = self.client.get(key)
            if bins['age'] != i + 2:
                assert True == False

        assert True == True

    def test_query_apply_with_none_set(self):
        """
        Invoke query_apply() with correct policy
        """
        policy = {'timeout': 1000}
        query_id = self.client.query_apply("test", None, p.between("age", 1, 5), "bin_lua",
                                         "mytransform", ['age', 2], policy)

        time.sleep(0.1)
        while True:
            response = self.client.job_info(query_id, aerospike.JOB_QUERY)
            if response['status'] != aerospike.JOB_STATUS_INPROGRESS:
                break
            time.sleep(0.1)

        for i in xrange(1, 5):
            key = ('test', 'demo', i)
            (key, meta, bins) = self.client.get(key)
            if bins['age'] != i:
                assert True == False

        assert True == True

    def test_query_apply_with_incorrect_policy(self):
        """
        Invoke query_apply() with incorrect policy
        """
        policy = {
            'timeout': 0.5
        }
        try:
            query_id = self.client.query_apply("test", "demo", ["age", 1, 5], "bin_lua", "mytransform", ['age', 2], policy)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "timeout is invalid"

    def test_query_apply_with_incorrect_ns_set(self):
        """
        Invoke query_apply() with incorrect ns and set
        """
        try:
            query_id = self.client.query_apply("test1", "demo1", ["age", 1, 5], "bin_lua", "mytransform", ['age', 2])

        except NamespaceNotFound as exception:
            assert exception.code == 20L
        except ServerError as exception:
            assert exception.code == 1L

    def test_query_apply_with_incorrect_module_name(self):
        """
        Invoke query_apply() with incorrect module name
        """
        query_id = self.client.query_apply("test", "demo", p.between("age", 1,
            5), "bin_lua_incorrect", "mytransform", ['age', 2])

        time.sleep(0.1)
        while True:
            response = self.client.job_info(query_id, aerospike.JOB_QUERY)
            if response['status'] != aerospike.JOB_STATUS_INPROGRESS:
                break
            time.sleep(0.1)

        for i in xrange(1, 5):
            key = ('test', 'demo', i)
            (key, meta, bins) = self.client.get(key)
            if bins['age'] != i:
                assert True == False
            else:
                assert True == True

    def test_query_apply_with_incorrect_function_name(self):
        """
        Invoke query_apply() with incorrect function name
        """
        query_id = self.client.query_apply("test", "demo", p.between("age", 1,
            5), "bin_lua", "mytransform_incorrect", ['age', 2])

        time.sleep(0.1)
        while True:
            response = self.client.job_info(query_id, aerospike.JOB_QUERY)
            if response['status'] != aerospike.JOB_STATUS_INPROGRESS:
                break
            time.sleep(0.1)

        for i in xrange(1, 5):
            key = ('test', 'demo', i)
            (key, meta, bins) = self.client.get(key)
            if bins['age'] != i:
                assert True == False
            else:
                assert True == True

    def test_query_apply_with_ns_set_none(self):
        """
        Invoke query_apply() with ns and set as None
        """
        with pytest.raises(TypeError) as typeError:
            query_id = self.client.query_apply(None, None, p.between("age", 1, 5), "bin_lua",
                                             "mytransform", ['age', 2])

        assert "query_apply() argument 1 must be string, not None" in typeError.value

    def test_query_apply_with_module_function_none(self):
        """
        Invoke query_apply() with None module function
        """

        try:
            query_id = self.client.query_apply("test", "demo", p.between("age",
                1, 5), None, None, ['age', 2])

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == "Module name should be string"

    def test_query_apply_with_extra_argument(self):
        """
        Invoke query_apply() with extra argument
        """
        policy = {'timeout': 1000}
        with pytest.raises(TypeError) as typeError:
            query_id = self.client.query_apply("test", "demo", p.between("age",
                1, 5), "bin_lua", "mytransform_incorrect", ['age', 2], policy, "")

        assert "query_apply() takes at most 7 arguments (8 given)" in typeError.value

    def test_query_apply_with_argument_is_string(self):
        """
        Invoke query_apply() with arguments as string
        """
        try:
            query_id = self.client.query_apply("test", "demo", p.between("age",
                1, 5), "bin_lua", "mytransform", "")

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Arguments should be a list"

    def test_query_apply_with_argument_is_none(self):
        """
        Invoke query_apply() with arguments as None
        """
        try:
            query_id = self.client.query_apply("test", "demo", p.between("age",
                1, 5), "bin_lua", "mytransform", None)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Arguments should be a list"

    def test_query_apply_with_extra_call_to_lua(self):
        """
        Invoke query_apply() with extra call to lua
        """
        query_id = self.client.query_apply("test", "demo", p.between("age", 1,
            5), "bin_lua","mytransform", ['age', 2, 3])

        #time.sleep(2)

        time.sleep(0.1)
        while True:
            response = self.client.job_info(query_id, aerospike.JOB_QUERY)
            if response['status'] != aerospike.JOB_STATUS_INPROGRESS:
                break
            time.sleep(0.1)

        for i in xrange(1, 5):
            key = ('test', 'demo', i)
            (key, meta, bins) = self.client.get(key)
            if bins['age'] != i + 2:
                assert True == False

        assert True == True

    def test_query_apply_with_extra_parameter_in_lua(self):
        """
        Invoke query_apply() with extra parameter in lua
        """
        query_id = self.client.query_apply("test", "demo", p.between("age", 1,
            5), "bin_lua", "mytransformextra", ['age', 2])

        #time.sleep(2)

        time.sleep(0.1)
        while True:
            response = self.client.job_info(query_id, aerospike.JOB_QUERY)
            if response['status'] != aerospike.JOB_STATUS_INPROGRESS:
                break
            time.sleep(0.1)

        for i in xrange(1, 5):
            key = ('test', 'demo', i)
            (key, meta, bins) = self.client.get(key)
            if bins['age'] != i + 2:
                assert True == False

        assert True == True

    def test_query_apply_with_less_parameter_in_lua(self):
        """
        Invoke query_apply() with less parameter in lua
        """
        query_id = self.client.query_apply("test", "demo", p.between("age", 1,
            5), "bin_lua", "mytransformless", ['age', 2])

        #time.sleep(2)

        time.sleep(0.1)
        while True:
            response = self.client.job_info(query_id, aerospike.JOB_QUERY)
            if response['status'] != aerospike.JOB_STATUS_INPROGRESS:
                break
            time.sleep(0.1)

        for i in xrange(1, 5):
            key = ('test', 'demo', i)
            (key, meta, bins) = self.client.get(key)
            if bins['age'] != i:
                assert True == False
            else:
                assert True == True

    def test_query_apply_unicode_input(self):
        """
        Invoke query_apply() with unicode udf
        """
        query_id = self.client.query_apply("test", "demo", p.between("age", 1,
            5), u"bin_lua", u"mytransform", ['age', 2])

        time.sleep(0.1)
        while True:
            response = self.client.job_info(query_id, aerospike.JOB_QUERY)
            if response['status'] != aerospike.JOB_STATUS_INPROGRESS:
                break
            time.sleep(0.1)

        for i in xrange(1, 5):
            key = ('test', 'demo', i)
            (key, meta, bins) = self.client.get(key)
            if bins['age'] != i + 2:
                assert True == False

        assert True == True

    def test_query_apply_with_correct_parameters_without_connection(self):
        """
        Invoke query_apply() with correct parameters without connection
        """
        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)

        try:
            query_id = client1.query_apply("test", "demo", p.between("age", 1,
                5), "bin_lua", "mytransform", ['age', 2])

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'

    def test_query_apply_with_incorrect_module_type(self):
        """
        Invoke query_apply() with incorrect module type
        """
        query_id = self.client.query_apply("test", "demo", p.between("age", 1,
            5), "bin_lua", "mytransform", ['age', 2])

        try:
            time.sleep(0.1)
            response = self.client.job_info(query_id, "aggregate")
        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Module can have only two values: aerospike.JOB_SCAN or aerospike.JOB_QUERY"



# -*- coding: utf-8 -*-

import pytest
import sys
import time
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestQueryRole(TestBaseClass):

    pytestmark = pytest.mark.skipif(
        TestBaseClass().get_hosts()[1] == None,
        reason="No user specified, may be not secured cluster.")

    def setup_method(self, method):

        """
        Setup method
        """
        hostlist, user, password = TestBaseClass().get_hosts()
        config = {
                "hosts": hostlist
                }
        self.client = aerospike.client(config).connect( user, password )
        try:
            self.client.admin_drop_role("usr-sys-admin")
        except:
            pass
        usr_sys_admin_privs =  [
            {"code": aerospike.PRIV_USER_ADMIN},
            {"code": aerospike.PRIV_SYS_ADMIN}]
        try:
            self.client.admin_drop_role("usr-sys-admin-test")
        except:
            pass
        self.client.admin_create_role("usr-sys-admin-test", usr_sys_admin_privs)

        self.delete_users = []
        time.sleep(1)

    def teardown_method(self, method):

        """
        Teardown method
        """

        policy = {}

        self.client.admin_drop_role("usr-sys-admin-test")
        self.client.close()

    def test_admin_query_role_no_parameters(self):
        """
        Query role with no parameters
        """
        with pytest.raises(TypeError) as typeError:
            self.client.admin_query_role()

        assert "Required argument 'role' (pos 1) not found" in typeError.value

    def test_admin_query_role_positive(self):
        """
            Query role positive
        """
        roles = self.client.admin_query_role("usr-sys-admin-test")
        assert roles == [{'code': 0, 'ns': '', 'set': ''}, {'code': 1, 'ns': '', 'set': ''}]

    def test_admin_query_role_positive_with_policy(self):
        """
            Query role positive policy
        """
        roles = self.client.admin_query_role("usr-sys-admin-test", {'timeout': 1000})
        assert roles == [{'code': 0, 'ns': '', 'set': ''}, {'code': 1, 'ns': '', 'set': ''}]

    def test_admin_query_role_incorrect_role_name(self):
        """
            Incorrect role name
        """
        try:
            self.client.admin_query_role("usr-sys-admin-test-non-existent", {'timeout': 1000})

        except InvalidRole as exception:
            assert exception.code == 70
            assert exception.msg == "AEROSPIKE_INVALID_ROLE"

    def test_admin_query_role_incorrect_role_type(self):
        """
            Incorrect role type
        """
        try:
            self.client.admin_query_role(None, {'timeout': 1000})

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Role name should be a string"



# -*- coding: utf-8 -*-

import pytest
import sys
from test_base_class import TestBaseClass
try:
    from collections import Counter
except ImportError:
    from counter26 import Counter

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestGetMany(TestBaseClass):
    def setup_class(cls):
        """
        Setup method.
        """
        TestGetMany.hostlist, TestGetMany.user, TestGetMany.password = TestBaseClass.get_hosts()
        config = {'hosts': TestGetMany.hostlist}
        if TestGetMany.user == None and TestGetMany.password == None:
            TestGetMany.client = aerospike.client(config).connect()
        else:
            TestGetMany.client = aerospike.client(config).connect(TestGetMany.user,
                                                                  TestGetMany.password)

    def teardown_class(cls):
        TestGetMany.client.close()

    def setup_method(self, method):
        self.keys = []

        for i in xrange(5):
            key = ('test', 'demo', i)
            rec = {'name': 'name%s' % (str(i)), 'age': i}
            TestGetMany.client.put(key, rec)
            self.keys.append(key)
        key = ('test', 'demo', 'float_value')
        TestGetMany.client.put(key, {"float_value": 4.3})
        self.keys.append(key)

    def teardown_method(self, method):
        """
        Teardown method.
        """
        for i in xrange(5):
            key = ('test', 'demo', i)
            TestGetMany.client.remove(key)
        key = ('test', 'demo', 'float_value')
        TestGetMany.client.remove(key)

    def test_get_many_without_any_parameter(self):

        with pytest.raises(TypeError) as typeError:
            TestGetMany.client.get_many()

        assert "Required argument 'keys' (pos 1) not found" in typeError.value

    def test_get_many_without_policy(self):

        records = TestGetMany.client.get_many(self.keys)

        assert type(records) == list
        assert len(records) == 6

    def test_get_many_with_proper_parameters(self):

        records = TestGetMany.client.get_many(self.keys, {'timeout': 30})

        assert type(records) == list
        assert len(records) == 6
        assert Counter([x[0][2] for x in records]) == Counter([0, 1, 2, 3,
            4, 'float_value'])
        assert records[5][2] == {'float_value': 4.3}

    def test_get_many_with_none_policy(self):

        records = TestGetMany.client.get_many(self.keys, None)

        assert type(records) == list
        assert len(records) == 6
        assert Counter([x[0][2] for x in records]) == Counter([0, 1, 2, 3,
            4, 'float_value'])
        assert records[5][2] == {'float_value': 4.3}

    def test_get_many_with_none_keys(self):

        try:
            TestGetMany.client.get_many( None, {} )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Keys should be specified as a list or tuple."

    def test_get_many_with_non_existent_keys(self):

        self.keys.append(('test', 'demo', 'non-existent'))

        records = TestGetMany.client.get_many(self.keys)

        assert type(records) == list
        assert len(records) == 7
        assert Counter([x[0][2] for x in records]) == Counter([0, 1, 2, 3,
            4, 'non-existent', 'float_value'])
        for x in records:
            if x[0][2] == 'non-existent':
                assert x[2] == None

    def test_get_many_with_all_non_existent_keys(self):

        keys = [('test', 'demo', 'key')]

        records = TestGetMany.client.get_many(keys)

        assert len(records) == 1
        assert records == [(('test', 'demo', 'key',
            bytearray(b';\xd4u\xbd\x0cs\xf2\x10\xb6~\xa87\x930\x0e\xea\xe5v(]')), None, None)]

    def test_get_many_with_invalid_key(self):

        try:
            records = TestGetMany.client.get_many( "key" )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Keys should be specified as a list or tuple."

    def test_get_many_with_invalid_timeout(self):

        policies = { 'timeout' : 0.2 }
        try:
            records = TestGetMany.client.get_many(self.keys, policies)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "timeout is invalid"

    def test_get_many_with_initkey_as_digest(self):

        keys = []
        key = ("test", "demo", None, bytearray("asd;as[d'as;djk;uyfl"))
        rec = {'name': 'name1', 'age': 1}
        TestGetMany.client.put(key, rec)

        keys.append(key)

        key = ("test", "demo", None, bytearray("ase;as[d'as;djk;uyfl"))
        rec = {'name': 'name2', 'age': 2}
        TestGetMany.client.put(key, rec)

        keys.append(key)

        records = TestGetMany.client.get_many(keys)

        for key in keys:
            TestGetMany.client.remove(key)

        assert type(records) == list
        assert len(records) == 2
        i = 0
        for x in records:
            if i:
                assert x[0][3] == bytearray(b"ase;as[d'as;djk;uyfl")
            else:
                assert x[0][3] == bytearray(b"asd;as[d'as;djk;uyfl")
            i = i+1

    def test_get_many_with_non_existent_keys_in_middle(self):

        self.keys.append(('test', 'demo', 'some_key'))

        for i in xrange(15, 20):
            key = ('test', 'demo', i)
            rec = {'name': 'name%s' % (str(i)), 'age': i}
            TestGetMany.client.put(key, rec)
            self.keys.append(key)

        records = TestGetMany.client.get_many(self.keys)

        for i in xrange(15, 20):
            key = ('test', 'demo', i)
            TestGetMany.client.remove(key)

        assert type(records) == list
        assert len(records) == 12
        assert Counter([x[0][2] for x in records]) == Counter([0, 1, 2, 3,
            4, 'some_key', 15, 16, 17, 18, 19, 'float_value'])
        for x in records:
            if x[0][2] == 'some_key':
                assert x[2] == None

    def test_get_many_with_proper_parameters_without_connection(self):
        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)
        try:
            records = client1.get_many( self.keys, { 'timeout': 20 } )

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'

    def test_get_many_with_use_batch_direct(self):

        config = {'hosts': TestGetMany.hostlist, 'policies': {'use_batch_direct':
            True}}
        if TestGetMany.user == None and TestGetMany.password == None:
            client_batch_direct = aerospike.client(config).connect()
        else:
            client_batch_direct = aerospike.client(config).connect(TestGetMany.user,
                                                                  TestGetMany.password)

        records = client_batch_direct.get_many(self.keys, {'timeout': 30})

        assert type(records) == list
        assert len(records) == 6
        assert Counter([x[0][2] for x in records]) == Counter([0, 1, 2, 3,
        4, 'float_value'])
        assert records[5][2] == {'float_value': 4.3}


        client_batch_direct.close()



# -*- coding: utf-8 -*-

import pytest
import sys
import time
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestQueryRoles(TestBaseClass):

    pytestmark = pytest.mark.skipif(
        TestBaseClass().get_hosts()[1] == None,
        reason="No user specified, may be not secured cluster.")

    def setup_method(self, method):

        """
        Setup method
        """
        hostlist, user, password = TestBaseClass().get_hosts()
        config = {
                "hosts": hostlist
                }
        self.client = aerospike.client(config).connect( user, password )
        try:
            self.client.admin_drop_role("usr-sys-admin")
        except:
            pass
        usr_sys_admin_privs =  [
            {"code": aerospike.PRIV_USER_ADMIN},
            {"code": aerospike.PRIV_SYS_ADMIN}]
        try:
            self.client.admin_drop_role("usr-sys-admin-test")
        except:
            pass
        self.client.admin_create_role("usr-sys-admin-test", usr_sys_admin_privs)
        self.delete_users = []
        time.sleep(1)

    def teardown_method(self, method):

        """
        Teardown method
        """
        policy = {}

        self.client.admin_drop_role("usr-sys-admin-test")
        self.client.close()

    def test_admin_query_roles_positive(self):
        """
            Query roles positive
        """
        roles = self.client.admin_query_roles()

        flag = 0
        assert roles['usr-sys-admin-test'] == [{'code': 0, 'ns': '', 'set': ''}, {'code': 1, 'ns': '', 'set': ''}]

    def test_admin_query_roles_positive_with_policy(self):
        """
            Query roles positive policy
        """
        roles = self.client.admin_query_roles({'timeout': 1000})

        flag = 0
        assert roles['usr-sys-admin-test'] == [{'code': 0, 'ns': '', 'set': ''}, {'code': 1, 'ns': '', 'set': ''}]

    def test_admin_query_roles_incorrect_policy(self):
        """
            Query roles incorrect policy
        """
        try:
            roles = self.client.admin_query_roles({'timeout': 0.2})

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'timeout is invalid'



# -*- coding: utf-8 -*-
import pytest
import time
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")


class TestIsconnected(object):
    def setup_method(self, method):
        """
        Setup method.
        """

    def teardown_method(self, method):
        """
        Teardoen method.
        """

    def test_isconnected_positive(self):
        """
        Invoke is_connected() positive.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(user, password)
        assert self.client.is_connected() == True
        self.client.close()

    def test_isconnected_beforeconnect(self):
        """
        Invoke is_connected() before connect
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        self.client = aerospike.client(config)
        assert self.client.is_connected() == False
        if user == None and password == None:
            self.client.connect()
        else:
            self.client.connect(user, password)
        assert self.client.is_connected() == True
        self.client.close()

    def test_isconnected_afterclose(self):
        """
        Invoke is_connected() after close.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(user, password)

        self.client.close()
        assert self.client.is_connected() == False



# -*- coding: utf-8 -*-

import pytest
import sys
import time
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestChangePassword(TestBaseClass):

    pytestmark = pytest.mark.skipif(
        TestBaseClass().get_hosts()[1] == None,
        reason="No user specified, may be not secured cluster.")

    def setup_method(self, method):
        """
        Setup method
        """
        hostlist, user, password = TestBaseClass().get_hosts()
        config = {"hosts": hostlist}
        self.client = aerospike.client(config).connect(user, password)

        try:
            self.client.admin_create_user( "testchangepassworduser", "aerospike", ["read"], {})
            time.sleep(2)
        except UserExistsError:
            pass # we are good, no such role exists
        self.delete_users = []

    def teardown_method(self, method):
        """
        Teardown method
        """

        self.client.admin_drop_user( "testchangepassworduser" )

        self.client.close()

    def test_change_password_without_any_parameters(self):

        with pytest.raises(TypeError) as typeError:
            status = self.client.admin_change_password()

        assert "Required argument 'user' (pos 1) not found" in typeError.value

    def test_change_password_with_proper_parameters(self):

        user = "testchangepassworduser"
        config = {"hosts": TestChangePassword.hostlist}
        self.clientreaduser = aerospike.client(config).connect(user,
                                                               "aerospike")

        policy = {}
        password = "newpassword"

        status = self.clientreaduser.admin_change_password( user, password )

        assert status == 0

        config = {
                "hosts": TestChangePassword.hostlist
                }
        try:
            self.clientreaduserwrong = aerospike.client(config).connect( user, "aerospike" )

        except InvalidPassword as exception:
            assert exception.code == 62 
            assert exception.msg == None
        except ClientError as exception:
            assert exception.code == -1
            assert exception.msg == "Failed to seed cluster"

        self.clientreaduserright = aerospike.client(config).connect(
            user, "newpassword")

        assert self.clientreaduserright != None

        self.clientreaduserright.close()
        self.clientreaduser.close()

    def test_change_password_with_invalid_timeout_policy_value(self):

        policy = {'timeout': 0.1}
        user = "testchangepassworduser"
        password = "newpassword"

        try:
            status = self.client.admin_change_password( user, password, policy )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "timeout is invalid"

    def test_change_password_with_proper_timeout_policy_value(self):

        user = "testchangepassworduser"
        config = {"hosts": TestChangePassword.hostlist}
        self.clientreaduser = aerospike.client(config).connect(user,
                                                               "aerospike")

        policy = {'timeout': 100}
        password = "newpassword"

        status = self.clientreaduser.admin_change_password( user, password, policy )

        assert status == 0

        config = {
                "hosts": TestChangePassword.hostlist
                }

        try:
            self.clientreaduserwrong = aerospike.client(config).connect( user, "aerospike" )

        except InvalidPassword as exception:
            assert exception.code == 62 
            assert exception.msg == None
        except ClientError as exception:
            assert exception.code == -1
            assert exception.msg == "Failed to seed cluster"

        self.clientreaduserright = aerospike.client(config).connect(
            user, "newpassword")

        assert self.clientreaduserright != None

        self.clientreaduserright.close()
        self.clientreaduser.close()

    def test_change_password_with_none_username(self):

        policy = {}
        user = None
        password = "newpassword"

        try:
            status = self.client.admin_change_password( user, password, policy )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Username should be a string"

    def test_change_password_with_none_password(self):

        policy = {}
        user = "testchangepassworduser"
        password = None

        try:
            status = self.client.admin_change_password( user, password, policy )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Password should be a string"

    def test_change_password_with_non_existent_user(self):

        policy = {}
        user = "readwriteuser"
        password = "newpassword"

        try:
            status = self.client.admin_change_password( user, password, policy )

        except InvalidUser as exception:
            assert exception.code == 60
            assert exception.msg == "AEROSPIKE_INVALID_USER"

    def test_change_password_with_too_long_password(self):

        user = "testchangepassworduser"
        config = {"hosts": TestChangePassword.hostlist}
        self.clientreaduser = aerospike.client(config).connect(user,
                                                               "aerospike")

        policy = {'timeout': 100}
        password = "password" * 1000

        status = self.clientreaduser.admin_change_password( user, password, policy )

        assert status == 0

        config = {
                "hosts": TestChangePassword.hostlist
                }

        try:
            self.clientreaduserwrong = aerospike.client(config).connect( user, "aerospike" )

        except InvalidPassword as exception:
            assert exception.code == 62 
            assert exception.msg == None
        except ClientError as exception:
            assert exception.code == -1
            assert exception.msg == "Failed to seed cluster"

        self.clientreaduserright = aerospike.client(config).connect(user,
                                                                    password)

        assert self.clientreaduserright != None

        self.clientreaduserright.close()
        self.clientreaduser.close()



# -*- coding: utf-8 -*-

import pytest
import sys
import cPickle as pickle
import time
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestInfo(object):
    def setup_class(cls):
        """
        Setup class.
        """
        TestInfo.hostlist, user, password = TestBaseClass.get_hosts()
        config = {
                'hosts': TestInfo.hostlist
                }
        TestInfo.config = config
        if user == None and password == None:
            TestInfo.client = aerospike.client(config).connect()
        else:
            TestInfo.client = aerospike.client(config).connect(user, password)

    def teardown_class(cls):
        """
        Teardoen class.
        """

        TestInfo.client.close()

    def test_info_for_statistics(self):

        request = "statistics"

        nodes_info = TestInfo.client.info(request, TestInfo.config['hosts'])

        assert nodes_info != None

        assert type(nodes_info) == dict

    def test_info_positive_for_namespace(self):
        """
        Test info positive for namespace
        """
        key = ('test', 'demo', 'list_key')

        rec = {'names': ['John', 'Marlen', 'Steve']}

        TestInfo.client.put(key, rec)
        response = TestInfo.client.info('namespaces', TestInfo.config['hosts'])
        TestInfo.client.remove(key)
        flag = 0
        for keys in response.keys():
            for value in response[keys]:
                if value != None:
                    if 'test' in value:
                        flag = 1
        if flag:
            assert True == True
        else:
            assert True == False

    def test_info_positive_for_sets(self):
        """
        Test info positive for sets
        """
        key = ('test', 'demo', 'list_key')

        rec = {'names': ['John', 'Marlen', 'Steve']}

        TestInfo.client.put(key, rec)
        response = TestInfo.client.info('sets', TestInfo.config['hosts'])
        TestInfo.client.remove(key)
        flag = 0
        for keys in response.keys():
            for value in response[keys]:
                if value != None:
                    if 'demo' in value:
                        flag = 1
        if flag:
            assert True == True
        else:
            assert True == False

    def test_info_positive_for_bins(self):
        """
        Test info positive for bins
        """
        key = ('test', 'demo', 'list_key')

        rec = {'names': ['John', 'Marlen', 'Steve']}

        TestInfo.client.put(key, rec)
        response = TestInfo.client.info('bins', TestInfo.config['hosts'])
        TestInfo.client.remove(key)
        flag = 0
        for keys in response.keys():
            for value in response[keys]:
                if value != None:
                    if 'names' in value:
                        flag = 1
        if flag:
            assert True == True
        else:
            assert True == False

    def test_info_with_config_for_statistics(self):

        request = u"statistics"

        config = [(127, 3000)]

        try:
            TestInfo.client.info(request, config)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Host address is of type incorrect"

    def test_info_with_config_for_statistics_and_policy(self):

        request = "statistics"
        policy = {'timeout': 1000}
        nodes_info = TestInfo.client.info(request, TestInfo.config['hosts'], policy)

        assert nodes_info != None

        assert type(nodes_info) == dict

    def test_info_for_invalid_request(self):

        request = "no_info"

        nodes_info = TestInfo.client.info(request, TestInfo.config['hosts'])

        assert type(nodes_info) == dict

        assert nodes_info.values() != None

    def test_info_with_none_request(self):

        request = None

        try:
            TestInfo.client.info(request, TestInfo.config['hosts'])

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == "Request must be a string"

    def test_info_without_parameters(self):

        with pytest.raises(TypeError) as typeError:
            nodes_info = TestInfo.client.info()

        assert "Required argument 'command' (pos 1) not found" in typeError.value

    def test_info_positive_for_sets_without_connection(self):
        """
        Test info positive for sets without connection
        """
        client1 = aerospike.client(TestInfo.config)
        try:
            response = client1.info('sets', TestInfo.config['hosts'])

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'




# -*- coding: utf-8 -*-

import pytest
import sys
from test_base_class import TestBaseClass
from aerospike import predicates as p

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestGeospatial(TestBaseClass):

    pytestmark = pytest.mark.skipif(
        TestBaseClass.has_geo_support() == False,
        reason="Server does not support geospatial data.")

    def setup_class(cls):
        """
        Setup method.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestGeospatial.client = aerospike.client(config).connect()
        else:
            TestGeospatial.client = aerospike.client(config).connect(user, password)
        TestGeospatial.client.index_geo2dsphere_create("test", "demo", "loc", "loc_index")

    def teardown_class(cls):
        TestGeospatial.client.index_remove('test', 'loc_index')
        TestGeospatial.client.close()

    def setup_method(self, method):

        self.keys = []
        for i in xrange(10):
            key = ('test', 'demo', i)
            lng = -122 + (0.2 * i)
            lat = 37.5 + (0.2 * i)
            self.geo_object = aerospike.GeoJSON({"type": "Point", "coordinates": [lng, lat] })
    
            TestGeospatial.client.put(key, {"loc": self.geo_object})
            self.keys.append(key)

    def teardown_method(self, method):
        """
        Teardown method.
        """
        for key in self.keys:
            TestGeospatial.client.remove(key)

    def test_geospatial_put_get_positive(self):
        """
            Perform a get and put with multiple bins including geospatial bin
        """

        key = ('test', 'demo', 'single_geo_put')
        geo_object_single = aerospike.GeoJSON({"type": "Point", "coordinates": [42.34, 58.62] })
        geo_object_dict = aerospike.GeoJSON({"type": "Point", "coordinates": [56.34, 69.62] })
        
        TestGeospatial.client.put(key, {"loc": geo_object_single, "int_bin": 2,
            "string_bin": "str", "dict_bin": {"a": 1, "b":2, "geo":
                geo_object_dict}})
        
        key, meta, bins = TestGeospatial.client.get(key)

        expected = {'loc': {'coordinates': [42.34, 58.62], 'type': 'Point'},
                "int_bin": 2, "string_bin": "str", "dict_bin": {"a": 1, "b": 2,
                  "geo": {'coordinates': [56.34, 69.62], 'type':
                        'Point'}}}
        for b in bins:
            assert b in expected

        TestGeospatial.client.remove(key)

    def test_geospatial_positive_query(self):
        """
            Perform a positive geospatial query for a polygon
        """
        records = []
        query = TestGeospatial.client.query("test", "demo")

        geo_object2 = aerospike.GeoJSON({"type": "Polygon", "coordinates": [[[-122.500000,
    37.000000],[-121.000000, 37.000000], [-121.000000, 38.080000],[-122.500000,
        38.080000], [-122.500000, 37.000000]]]})

        query.where(p.geo_within("loc", geo_object2.dumps()))

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)

        assert len(records) == 3
        expected = [{'loc': {'coordinates': [-122.0, 37.5], 'type': 'Point'}}, {'loc': {'coordinates': [-121.8, 37.7], 'type':
                'Point'}}, {'loc': {'coordinates': [-121.6, 37.9], 'type': 'Point'}}]
        for r in records:
            assert r in expected

    def test_geospatial_positive_query_outside_shape(self):
        """
            Perform a positive geospatial query for polygon where all points are
            outside polygon
        """
        records = []
        query = TestGeospatial.client.query("test", "demo")

        geo_object2 = aerospike.GeoJSON({"type": "Polygon", "coordinates": [[[-126.500000,
    37.000000],[-124.000000, 37.000000], [-124.000000, 38.080000],[-126.500000,
        38.080000], [-126.500000, 37.000000]]]})

        query.where(p.geo_within("loc", geo_object2.dumps()))

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)

        assert len(records) == 0

    def test_geospatial_positive_query_without_set(self):
        """
            Perform a positive geospatial query for a polygon without a set
        """
        keys = []
        for i in xrange(1, 10):
            key = ('test', None, i)
            lng = -122 + (0.2 * i)
            lat = 37.5 + (0.2 * i)
            geo_object = aerospike.GeoJSON({"type": "Point", "coordinates": [lng, lat] })
    
            TestGeospatial.client.put(key, {"loc": geo_object})
            keys.append(key)

        TestGeospatial.client.index_geo2dsphere_create("test", None, "loc", "loc_index_no_set")
        records = []
        query = TestGeospatial.client.query("test", None)

        geo_object2 = aerospike.GeoJSON({"type": "Polygon", "coordinates": [[[-122.500000,
    37.000000],[-121.000000, 37.000000], [-121.000000, 38.080000],[-122.500000,
        38.080000], [-122.500000, 37.000000]]]})

        query.where(p.geo_within("loc", geo_object2.dumps()))

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)

        TestGeospatial.client.index_remove('test', 'loc_index_no_set')
        for key in keys:
            TestGeospatial.client.remove(key)

        assert len(records) == 2
        expected = [{'loc': {'coordinates': [-121.8, 37.7], 'type': 'Point'}}, {'loc': {'coordinates': [-121.6, 37.9], 'type': 'Point'}}]
        for r in records:
            assert r in expected

    def test_geospatial_positive_query_for_circle(self):
        """
            Perform a positive geospatial query for a circle
        """
        records = []
        query = TestGeospatial.client.query("test", "demo")

        geo_object2 = aerospike.GeoJSON({"type": "AeroCircle", "coordinates": [[-122.0, 37.5], 250.2]})

        query.where(p.geo_within("loc", geo_object2.dumps()))

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)

        assert len(records) == 1
        assert records == [{'loc': {'coordinates': [-122.0, 37.5], 'type': 'Point'}}]

    def test_geospatial_operate_positive(self):
        """
            Perform an operate operation with geospatial bin
        """

        geo_object_operate = aerospike.GeoJSON({"type": "Point", "coordinates": [43.45, 56.75] })
        key = ('test', 'demo', 'single_geo_operate')
        list = [{
            "op": aerospike.OPERATOR_WRITE,
            "bin": "write_bin",
            "val": {"no": geo_object_operate}
            }, {"op": aerospike.OPERATOR_READ,
                "bin": "write_bin"}]

        key, meta, bins = TestGeospatial.client.operate(key, list)

        assert bins == {'write_bin': {'no': {'coordinates': [43.45, 56.75],
            'type': 'Point'}}}

        TestGeospatial.client.remove(key)

    def test_geospatial_object_not_dict_or_string(self):
        """
            The geospatial object is not a dictionary or string
        """
        try:
            geo_object_wrong = aerospike.GeoJSON(1)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'Geospatial data should be a dictionary or raw GeoJSON string'

    def test_geospatial_object_non_json_serialziable_string(self):
        """
            The geospatial object is not a json serializable string
        """
        try:
            geo_object_wrong = aerospike.GeoJSON("abc")

        except ClientError as exception:
            assert exception.code == -1
            assert exception.msg == 'String is not GeoJSON serializable'

    def test_geospatial_wrap_positive(self):
        """
            Perform a positive wrap on geospatial data
        """
        self.geo_object.wrap({"type": "Polygon", "coordinates": [[[-122.500000, 
            37.000000],[-121.000000, 37.000000], [-121.000000, 38.080000],[-122.500000,
            38.080000], [-122.500000, 37.000000]]]})
        assert self.geo_object.unwrap() == {'coordinates': [[[-122.5, 37.0], [-121.0, 37.0], [-121.0, 38.08],
            [-122.5, 38.08], [-122.5, 37.0]]], 'type': 'Polygon'}

    def test_geospatial_object_wrap_non_dict(self):
        """
            The geospatial object provided to wrap() is not a dictionary
        """
        try:
            self.geo_object.wrap("abc")
        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'Geospatial data should be a dictionary or raw GeoJSON string'

    def test_geospatial_wrap_positive_with_query(self):
        """
            Perform a positive wrap on geospatial data followed by a query
        """
        geo_object_wrap = aerospike.GeoJSON({"type": "Polygon", "coordinates": [[[-124.500000,
    37.000000],[-125.000000, 37.000000], [-121.000000, 38.080000],[-122.500000,
        38.080000], [-124.500000, 37.000000]]]})

        geo_object_wrap.wrap({"type": "Polygon", "coordinates": [[[-122.500000, 
            37.000000],[-121.000000, 37.000000], [-121.000000, 38.080000],[-122.500000,
            38.080000], [-122.500000, 37.000000]]]})
        assert geo_object_wrap.unwrap() == {'coordinates': [[[-122.5, 37.0], [-121.0, 37.0], [-121.0, 38.08],
            [-122.5, 38.08], [-122.5, 37.0]]], 'type': 'Polygon'}
        
	records = []
        query = TestGeospatial.client.query("test", "demo")
        query.where(p.geo_within("loc", geo_object_wrap.dumps()))

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)

        assert len(records) == 3
        expected = [{'loc': {'coordinates': [-122.0, 37.5], 'type': 'Point'}}, {'loc': {'coordinates': [-121.8, 37.7], 'type':
                'Point'}}, {'loc': {'coordinates': [-121.6, 37.9], 'type': 'Point'}}]
        for r in records:
            assert r in expected

    def test_geospatial_loads_positive(self):
        """
            Perform a positive loads on geoJSON raw string
        """
        self.geo_object.loads('{"type": "Polygon", "coordinates": [[[-122.500000, 37.000000], [-121.000000, 37.000000], [-121.000000, 38.080000],[-122.500000, 38.080000], [-122.500000, 37.000000]]]}')

        assert self.geo_object.unwrap() == {'coordinates': [[[-122.5, 37.0], [-121.0, 37.0], [-121.0, 38.08],
            [-122.5, 38.08], [-122.5, 37.0]]], 'type': 'Polygon'}

    def test_geospatial_object_loads_non_dict(self):
        """
            The geospatial object provided to loads() is not a dictionary
        """
        try:
            self.geo_object.loads('{"abc"}')
        except ClientError as exception:
            assert exception.code == -1
            assert exception.msg == 'String is not GeoJSON serializable'

    def test_geospatial_loads_positive_with_query(self):
        """
            Perform a positive loads on geoJSON raw string followed by a query
        """
        geo_object_loads = aerospike.GeoJSON({"type": "Polygon", "coordinates": [[[-124.500000,
    37.000000],[-125.000000, 37.000000], [-121.000000, 38.080000],[-122.500000,
        38.080000], [-124.500000, 37.000000]]]})

        geo_object_loads.loads('{"type": "Polygon", "coordinates": [[[-122.500000, 37.000000], [-121.000000, 37.000000], [-121.000000, 38.080000],[-122.500000, 38.080000], [-122.500000, 37.000000]]]}')

        assert geo_object_loads.unwrap() == {'coordinates': [[[-122.5, 37.0], [-121.0, 37.0], [-121.0, 38.08],
            [-122.5, 38.08], [-122.5, 37.0]]], 'type': 'Polygon'}

	records = []
        query = TestGeospatial.client.query("test", "demo")
        query.where(p.geo_within("loc", geo_object_loads.dumps()))

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)

        assert len(records) == 3
        expected = [{'loc': {'coordinates': [-122.0, 37.5], 'type': 'Point'}}, {'loc': {'coordinates': [-121.8, 37.7], 'type':
                'Point'}}, {'loc': {'coordinates': [-121.6, 37.9], 'type': 'Point'}}]
        for r in records:
            assert r in expected

    def test_geospatial_dumps_positive(self):
        """
            Perform a positive dumps. Verify using str
        """
        assert self.geo_object.dumps() == '{"type": "Point", "coordinates": [-120.2, 39.3]}'

        assert str(self.geo_object) == '{"type": "Point", "coordinates": [-120.2, 39.3]}'

    def test_geospatial_repr_positive(self):
        """
            Perform a positive repr. Verify using eval()
        """
	assert repr(self.geo_object) == '\'{"type": "Point", "coordinates": [-120.2, 39.3]}\''
        assert eval(repr(self.geo_object)) == '{"type": "Point", "coordinates": [-120.2, 39.3]}'


    def test_geospatial_put_get_positive_with_geodata(self):
        """
            Perform a get and put with multiple bins including geospatial bin
            using geodata method
        """

        key = ('test', 'demo', 'single_geo_put')
        geo_object_single = aerospike.geodata({"type": "Point", "coordinates": [42.34, 58.62] })
        geo_object_dict = aerospike.geodata({"type": "Point", "coordinates": [56.34, 69.62] })
        
        TestGeospatial.client.put(key, {"loc": geo_object_single, "int_bin": 2,
            "string_bin": "str", "dict_bin": {"a": 1, "b":2, "geo":
                geo_object_dict}})
        
        key, meta, bins = TestGeospatial.client.get(key)

        expected = {'loc': {'coordinates': [42.34, 58.62], 'type': 'Point'},
                "int_bin": 2, "string_bin": "str", "dict_bin": {"a": 1, "b": 2,
                  "geo": {'coordinates': [56.34, 69.62], 'type':
                        'Point'}}}
        for b in bins:
            assert b in expected

        TestGeospatial.client.remove(key)

    def test_geospatial_put_get_positive_with_geojson(self):
        """
            Perform a get and put with multiple bins including geospatial bin
            using geodata method
        """

        key = ('test', 'demo', 'single_geo_put')
        geo_object_single = aerospike.geojson('{"type": "Point", "coordinates": [42.34, 58.62] }')
        geo_object_dict = aerospike.geojson('{"type": "Point", "coordinates": [56.34, 69.62] }')
        
        TestGeospatial.client.put(key, {"loc": geo_object_single, "int_bin": 2,
            "string_bin": "str", "dict_bin": {"a": 1, "b":2, "geo":
                geo_object_dict}})
        
        key, meta, bins = TestGeospatial.client.get(key)

        expected = {'loc': {'coordinates': [42.34, 58.62], 'type': 'Point'},
                "int_bin": 2, "string_bin": "str", "dict_bin": {"a": 1, "b": 2,
                  "geo": {'coordinates': [56.34, 69.62], 'type':
                        'Point'}}}
        for b in bins:
            assert b in expected

        TestGeospatial.client.remove(key)

    def test_geospatial_positive_query_with_geodata(self):
        """
            Perform a positive geospatial query for a polygon with geodata
        """
        records = []
        query = TestGeospatial.client.query("test", "demo")

        geo_object2 = aerospike.geodata({"type": "Polygon", "coordinates": [[[-122.500000,
    37.000000],[-121.000000, 37.000000], [-121.000000, 38.080000],[-122.500000,
        38.080000], [-122.500000, 37.000000]]]})

        query.where(p.geo_within("loc", geo_object2.dumps()))

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)

        assert len(records) == 3
        expected = [{'loc': {'coordinates': [-122.0, 37.5], 'type': 'Point'}}, {'loc': {'coordinates': [-121.8, 37.7], 'type':
                'Point'}}, {'loc': {'coordinates': [-121.6, 37.9], 'type': 'Point'}}]
        for r in records:
            assert r in expected

    def test_geospatial_positive_query_with_geojson(self):
        """
            Perform a positive geospatial query for a polygon with geojson
        """
        records = []
        query = TestGeospatial.client.query("test", "demo")

        geo_object2 = aerospike.geojson('{"type": "Polygon", "coordinates": [[[-122.500000, 37.000000], [-121.000000, 37.000000], [-121.000000, 38.080000],[-122.500000, 38.080000], [-122.500000, 37.000000]]]}')

        query.where(p.geo_within("loc", geo_object2.dumps()))

        def callback((key, metadata, record)):
            records.append(record)

        query.foreach(callback)

        assert len(records) == 3
        expected = [{'loc': {'coordinates': [-122.0, 37.5], 'type': 'Point'}}, {'loc': {'coordinates': [-121.8, 37.7], 'type':
                'Point'}}, {'loc': {'coordinates': [-121.6, 37.9], 'type': 'Point'}}]
        for r in records:
            assert r in expected

    def test_geospatial_2dindex_positive(self):
        """
            Perform a positive 2d index creation
        """
        status = TestGeospatial.client.index_remove('test', 'loc_index')

        assert status == 0

        status = TestGeospatial.client.index_geo2dsphere_create("test", "demo", "loc", "loc_index")

        assert status == 0

    def test_geospatial_2dindex_positive_with_policy(self):
        """
            Perform a positive 2d index creation with policy
        """
        status = TestGeospatial.client.index_remove('test', 'loc_index')

        assert status == 0

        status = TestGeospatial.client.index_geo2dsphere_create("test", "demo", "loc", "loc_index", {"timeout": 2000})

        assert status == 0

    def test_geospatial_2dindex_set_length_extra(self):
        """
            Perform a 2d creation with set length exceeding limit
        """
        set_name = 'a'
        for i in xrange(100):
            set_name = set_name + 'a'
        status = TestGeospatial.client.index_remove('test', 'loc_index')

        assert status == 0
        try:
            status = TestGeospatial.client.index_geo2dsphere_create("test", set_name, "loc", "loc_index")
        except InvalidRequest as exception:
            assert exception.code == 4



# -*- coding: utf-8 -*-

import pytest
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class SomeClass(object):

    pass


class TestExists(TestBaseClass):
    def setup_class(cls):
        """
        Setup method.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestExists.client = aerospike.client(config).connect()
        else:
            TestExists.client = aerospike.client(config).connect(user, password)

    def teardown_class(cls):
        TestExists.client.close()

    def setup_method(self, method):

        self.keys = []

        for i in xrange(5):
            key = ('test', 'demo', i)
            rec = {'name': 'name%s' % (str(i)), 'age': i}
            TestExists.client.put(key, rec)
            self.keys.append(key)

        key = ('test', 'demo', 'list_key')
        self.keys.append(key)

        rec = {'names': ['John', 'Marlen', 'Steve']}

        TestExists.client.put(key, rec)

        key = ('test', 'demo', 'map_key')
        self.keys.append(key)

        rec = {'names': {'name': 'John', 'age': 24}}

        TestExists.client.put(key, rec)

        key = ('test', 'demo', 'list_map_key')
        self.keys.append(key)

        rec = {
            'names': ['John', 'Marlen', 'Steve'],
            'names_and_age': [{'name': 'John',
                               'age': 24}, {'name': 'Marlen',
                                            'age': 25}]
        }

        TestExists.client.put(key, rec)

        obj1, obj2 = SomeClass(), SomeClass()

        key = ('test', 'demo', 'objects')
        self.keys.append(key)

        rec = {'objects': [pickle.dumps(obj1), pickle.dumps(obj2)]}
        TestExists.client.put(key, rec)

        key = ('test', 'demo', 'bytes_key')
        self.keys.append(key)

        rec = {'bytes': bytearray('John', 'utf-8')}
        TestExists.client.put(key, rec)

    def teardown_method(self, method):
        """
        Teardoen method.
        """

        for key in self.keys:
            TestExists.client.remove(key)

    def test_exists_with_no_paramters(self):
        """
            Invoke self() without any mandatory parameters.
        """
        with pytest.raises(TypeError) as typeError:
            TestExists.client.exists()

        assert "Required argument 'key' (pos 1) not found" in typeError.value

    def test_exists_with_only_key(self):
        """
            Invoke exists() with a key and not policy's dict.
        """
        key = ('test', 'demo', 1)

        key, meta = TestExists.client.exists(key)

        assert meta['gen'] != None
        assert meta['ttl'] != None

    def test_exists_with_key_and_policy(self):
        """
            Invoke exists() with a key and policy.
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'replica': aerospike.POLICY_REPLICA_MASTER,
            'consistency': aerospike.POLICY_CONSISTENCY_ONE
        }

        key, meta = TestExists.client.exists(key, policy)

        assert meta['gen'] != None
        assert meta['ttl'] != None

    def test_exists_with_policy_is_string(self):
        """
            Invoke exists() with a key and policy as string.
        """
        key = ('test', 'demo', 1)
        policy = ""

        try:
            key, meta = TestExists.client.exists( key, policy )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'policy must be a dict'

    def test_exists_with_timeout_is_string(self):
        """
            Invoke exists() with a key and timeout as string.
        """
        key = ('test', 'demo', 1)
        policy = {'timeout': "1000"}

        try:
            key, meta = TestExists.client.exists( key, policy )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'timeout is invalid'

    def test_exists_for_list_type_record(self):
        """
            Invoke exists() for list typed record.
        """
        key = ('test', 'demo', 'list_key')

        key, meta = TestExists.client.exists(key)

        assert meta['gen'] != None
        assert meta['ttl'] != None

    def test_exists_for_map_type_record(self):
        """
            Invoke exists() for map type record.
        """
        key = ('test', 'demo', 'map_key')

        key, meta = TestExists.client.exists(key)

        assert meta['gen'] != None
        assert meta['ttl'] != None

    def test_exists_for_list_and_map_type_combined(self):
        """
            Invoke exists() for list and map combined record.
        """
        key = ('test', 'demo', 'list_map_key')

        key, meta = TestExists.client.exists(key)

        assert meta['gen'] != None
        assert meta['ttl'] != None

    def test_exists_with_list_of_objects(self):
        """
            Invoke exists() for list of objects.
        """
        key = ('test', 'demo', 'objects')

        key, meta = TestExists.client.exists(key)

        assert meta['gen'] != None
        assert meta['ttl'] != None

    def test_exists_with_bytearray(self):
        """
            Invoke exists() for bytarray record.
        """
        key = ('test', 'demo', 'bytes_key')

        key, meta = TestExists.client.exists(key)

        assert meta['gen'] != None
        assert meta['ttl'] != None

    def test_exists_with_typed_key(self):
        """
            Invoke exists() with a string key and not policy's dict.
        """
        key = ('test', 'demo', '1asfew923987fas3g')

        try:
            key, meta = TestExists.client.exists( key )

            """
            We are making the api backward compatible. In case of RecordNotFound an
            exception will not be raised. Instead Ok response is returned withe the
            meta as None. This might change with further releases.
            """
        except RecordNotFound as exception:
            assert True == False
            assert exception.code == 2
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_NOT_FOUND'

        assert meta == None

    def test_exists_with_none_set(self):
        """
            Invoke exists() with None set in key tuple.
        """
        key = ('test', None, 2)

        try:
            key, meta = TestExists.client.exists( key )

            """
            We are making the api backward compatible. In case of RecordNotFound an
            exception will not be raised. Instead Ok response is returned withe the
            meta as None. This might change with further releases.
            """
        except RecordNotFound as exception:
            assert True == False
            assert exception.code == 2
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_NOT_FOUND'

        assert meta == None

    def test_exists_with_none_namespace(self):
        """
            Invoke exists() with None namespace in key tuple.
        """
        key = (None, 'demo', 2)

        try:
            key, meta = TestExists.client.exists( key )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'namespace must be a string'

    def test_exists_with_none_pk(self):
        """
            Invoke exists() with None as primary_key part of key tuple.
        """
        key = ('test', 'demo', None)

        try:
            key, meta  = TestExists.client.exists( key )

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'either key or digest is required'

    def test_exists_with_none_key(self):
        """
            Invoke exists() with None as a key.
        """
        try:
            key, meta  = TestExists.client.exists(None)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "key is invalid"

    def test_exists_key_type_list(self):
        """
            Invoke exists() with key specified as a list of ns, set and key/digest.
        """
        key = ['test', 'demo', '1']

        try:
            key, meta = TestExists.client.exists(key)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "key is invalid"

    def test_exists_with_non_existent_namespace(self):
        """
            Invoke exists() for non-existent namespace.
        """
        key = ('namespace', 'demo', 1)

        try:
            key, meta = TestExists.client.exists(key)

        except NamespaceNotFound as exception:
            assert exception.code == 20
            assert exception.msg == 'AEROSPIKE_ERR_NAMESPACE_NOT_FOUND'

    def test_exists_with_non_existent_set(self):
        """
            Invoke exists() for non-existent set.
        """
        key = ('test', 'set', 1)

        try:
            key, meta = TestExists.client.exists(key)

            """
            We are making the api backward compatible. In case of RecordNotFound an
            exception will not be raised. Instead Ok response is returned withe the
            meta as None. This might change with further releases.
            """
        except RecordNotFound as exception:
            assert True == False
            assert exception.code == 2
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_NOT_FOUND'

        assert meta == None

    def test_exists_with_non_existent_key(self):
        """
            Invoke exists() for non-existent key.
        """
        key = ('test', 'demo', 'non-existent')
        try:
            key, meta = TestExists.client.exists( key )

            """
            We are making the api backward compatible. In case of RecordNotFound an
            exception will not be raised. Instead Ok response is returned withe the
            meta as None. This might change with further releases.
            """
        except RecordNotFound as exception:
            assert True == False
            assert exception.code == 2
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_NOT_FOUND'

        assert meta == None

    def test_exists_with_only_key_without_connection(self):
        """
            Invoke exists() with a key and not policy's dict and no connection
        """
        key = ('test', 'demo', 1)
        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)

        try:
            key, meta = client1.exists( key )

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'



import unittest
from test_base_class import TestBaseClass

import aerospike
from aerospike.exception import *

config = {"hosts": [("127.0.0.1", 3000)]}

# count records
count = 0


def count_records((key, meta, rec)):
    global count
    count += 1


def count_records_false((key, meta, rec)):
    global count
    count += 1
    return False


def digest_only(key):
    return (key[0], key[1], None, key[3])


class KVTestCase(unittest.TestCase, TestBaseClass):
    def setup_class(cls):
        KVTestCase.hostlist, KVTestCase.user, KVTestCase.password = TestBaseClass.get_hosts(
        )

    def setUp(self):
        config = {"hosts": KVTestCase.hostlist}
        if KVTestCase.user == None and KVTestCase.password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(KVTestCase.user,
                                                           KVTestCase.password)

    def tearDown(self):
        self.client.close()

    def test_1(self):
        '''
        Using a single key, 
        '''

        global count

        key = ("test", "unittest", "1")

        # cleanup records
        def remove_record((key, meta, rec)):
            self.client.remove(key)

        self.client.scan("test", "unittest").foreach(remove_record)

        recIn = {
            "i": 1234,
            "s": "abcd",
            "b": bytearray("efgh", "utf-8"),
            "l": [1357, "aceg", bytearray("aceg", "utf-8"), [1, 3, 5, 7],
                  {"a": 1,
                   "c": 3,
                   "e": 5,
                   "g": 7}],
            "m": {
                "i": 2468,
                "s": "bdfh",
                "l": [2468, "bdfh", bytearray("bdfh", "utf-8")],
                "m": {"b": 2,
                      "d": 4,
                      "f": 6,
                      "h": 8}
            },
        }

        # create the record
        rc = self.client.put(key, recIn)
        self.assertEqual(rc, 0, 'wrong return code')

        # ensure existence
        (key, meta) = self.client.exists(key)
        self.assertTrue(meta is not None)

        # count records
        count = 0
        self.client.scan("test", "unittest").foreach(count_records)
        assert count == 1
        self.assertEqual(count, 1, 'set should have 1 record')

        # read it
        (key, meta, recOut) = self.client.get(key)
        self.assertEqual(recIn, recOut, 'records do not match')

        # create the record
        rc = self.client.put(key, {"hello": "world"})
        self.assertEqual(rc, 0, 'wrong return code')

        # augmented record
        recIn["hello"] = "world"

        # read it
        (key, meta, recOut) = self.client.get(key)
        self.assertEqual(recIn, recOut, 'records do not match')

        # remove it
        rc = self.client.remove(key)
        self.assertEqual(rc, 0, 'wrong return code')

        # ensure not existent
        try:
            (key, meta) = self.client.exists(key)
            """
            We are making the api backward compatible. In case of RecordNotFound an
            exception will not be raised. Instead Ok response is returned withe the
            meta as None. This might change with further releases.
            """
        except RecordNotFound as exception:
            assert True == False
            assert exception.code == 2

        assert meta == None

        # count records
        count = 0
        self.client.scan("test", "unittest").foreach(count_records)
        self.assertEqual(count, 0, 'set should be empty')

    def test_2(self):
        '''
        Using a single key, with digest only.
        '''

        global count

        key = ("test", "unittest", "1")

        # cleanup records
        def each_record((key, meta, rec)):
            self.client.remove(key)

        self.client.scan("test", "unittest").foreach(each_record)

        recIn = {
            "i": 1234,
            "s": "abcd",
            "b": bytearray("efgh", "utf-8"),
            "l": [1357, "aceg", bytearray("aceg", "utf-8"), [1, 3, 5, 7],
                  {"a": 1,
                   "c": 3,
                   "e": 5,
                   "g": 7}],
            "m": {
                "i": 2468,
                "s": "bdfh",
                "l": [2468, "bdfh", bytearray("bdfh", "utf-8")],
                "m": {"b": 2,
                      "d": 4,
                      "f": 6,
                      "h": 8}
            },
            'a': {u'aa': u'11'},
            'k': {u'kk': u'22'}
        }

        # create the record
        rc = self.client.put(key, recIn)
        self.assertEqual(rc, 0, 'wrong return code')

        # ensure existence
        (key, meta) = self.client.exists(key)
        self.assertTrue(meta is not None)

        # count records
        count = 0
        self.client.scan("test", "unittest").foreach(count_records)
        self.assertEqual(count, 1, 'set should have 1 record')

        # read it
        (key, meta, recOut) = self.client.get(digest_only(key))
        self.assertEqual(recIn, recOut, 'records do not match')

        # create the record
        rc = self.client.put(digest_only(key), {"hello": "world"})
        self.assertEqual(rc, 0, 'wrong return code')

        # augmented record
        recIn["hello"] = "world"

        # read it
        (key, meta, recOut) = self.client.get(digest_only(key))
        self.assertEqual(recIn, recOut, 'records do not match')

        # remove it
        rc = self.client.remove(digest_only(key))
        self.assertEqual(rc, 0, 'wrong return code')

        # ensure not existent
        try:
            (key, meta) = self.client.exists(digest_only(key))
            """
            We are making the api backward compatible. In case of RecordNotFound an
            exception will not be raised. Instead Ok response is returned withe the
            meta as None. This might change with further releases.
            """
        except RecordNotFound as exception:
            assert True == False
            assert exception.code == 2

        assert meta == None

        # count records
        count = 0
        self.client.scan("test", "unittest").foreach(count_records)
        self.assertEqual(count, 0, 'set should be empty')

    def test_3(self):
        """
        Using multiple keys
        """
        from aerospike import predicates as p
        from time import sleep
        global count

        for i in xrange(2):
            key = ('test', 'unittest', i)
            rec = {
                'name': 'name%s' % (str(i)),
                'addr': 'name%s' % (str(i)),
                'age': i,
                'no': i
            }
            self.client.put(key, rec)

        self.client.index_integer_create('test', 'unittest', 'age',
                                         'age_index', {})

        query = self.client.query('test', 'unittest')

        query.select("name", "age")
        count = 0
        query.where(p.between('age', 1, 3))

        query.foreach(count_records_false)

        self.assertEqual(count, 1, "foreach failed")

        for i in xrange(2):
            key = ('test', 'unittest', i)
            self.client.remove(key)

        self.client.index_remove('test', 'age_index', {})


suite = unittest.TestLoader().loadTestsFromTestCase(KVTestCase)



# -*- coding: utf-8 -*-

import pytest
import sys
import time
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class SomeClass(object):

    pass


class TestDropUser(TestBaseClass):

    pytestmark = pytest.mark.skipif(
        TestBaseClass().get_hosts()[1] == None,
        reason="No user specified, may be not secured cluster.")

    def setup_method(self, method):
        """
        Setup method.
        """
        hostlist, user, password = TestBaseClass().get_hosts()
        config = {'hosts': hostlist}
        TestDropUser.Me = self
        self.client = aerospike.client(config).connect(user, password)
        try:
            self.client.admin_drop_user("foo-test")
        except:
            pass

    def teardown_method(self, method):
        """
        Teardoen method.
        """
        self.client.close()

    def test_drop_user_with_no_parameters(self):
        """
            Invoke drop_user() without any mandatory parameters.
        """
        with pytest.raises(TypeError) as typeError:
            self.client.admin_drop_user()

        assert "Required argument 'user' (pos 1) not found" in typeError.value

    def test_drop_user_with_policy_none(self):
        """
            Invoke drop_user() with policy none
        """
        policy = None
        user = "foo-test"
        password = "foo1"
        roles = ["read", "read-write", "sys-admin"]

        status = self.client.admin_create_user( user, password, roles, policy )

        time.sleep(2)

        assert status == 0
        user_details = self.client.admin_query_user( user, policy )

        assert user_details == ['read', 'read-write', 'sys-admin']

        status = self.client.admin_drop_user( user, policy )

        assert status == 0

        try:
            user_details = self.client.admin_query_user( user )

        except InvalidUser as exception:
            assert exception.code == 60L
            assert exception.msg == 'AEROSPIKE_INVALID_USER'

    def test_drop_user_with_user_none(self):
        """
            Invoke drop_user() with policy none
        """
        policy = {'timeout': 1000}
        try:
            self.client.admin_drop_user( None, policy )

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == 'Username should be a string'

    def test_drop_user_positive(self):
        """
            Invoke drop_user() with correct arguments.
        """
        policy = {'timeout': 1000}
        user = "foo-test"
        password = "foo1"
        roles = ["read", "read-write", "sys-admin"]

        status = self.client.admin_create_user( user, password, roles, policy )

        time.sleep(1)

        assert status == 0
        user_details = self.client.admin_query_user( user, policy )

        assert user_details == ['read', 'read-write', 'sys-admin']
        status = self.client.admin_drop_user( user, policy )
        assert status == 0

        time.sleep(1)

        try:
            user_details = self.client.admin_query_user( user, policy )

        except InvalidUser as exception:
            assert exception.code == 60L
            assert exception.msg == 'AEROSPIKE_INVALID_USER'

    def test_drop_user_positive_without_policy(self):

        """
            Invoke drop_user() with correct arguments.
        """
        policy = {
            'timeout': 1000
        }
        user = "foo-test"
        password = "foo1"
        roles = ["read", "read-write", "sys-admin"]

        status = self.client.admin_create_user( user, password, roles, policy )

        time.sleep(1)

        assert status == 0
        user_details = self.client.admin_query_user( user, policy )

        assert user_details == ['read', 'read-write', 'sys-admin']
        status = self.client.admin_drop_user( user )
        assert status == 0

        time.sleep(1)

        try:
            user_details = self.client.admin_query_user( user, policy )

        except InvalidUser as exception:
            assert exception.code == 60L
            assert exception.msg == 'AEROSPIKE_INVALID_USER'

    def test_drop_user_negative(self):
        """
            Invoke drop_user() with non-existent user.
        """
        policy = {}
        user = "foo-test"
        password = "foo1"
        roles = ["read", "read-write", "sys-admin"]
        try:
            user_details = self.client.admin_query_user( user, policy )

        except InvalidUser as exception:
            assert exception.code == 60L
            assert exception.msg == 'AEROSPIKE_INVALID_USER'

        try:
            status = self.client.admin_drop_user( user )

        except InvalidUser as exception:
            assert exception.code == 60L
            assert exception.msg == 'AEROSPIKE_INVALID_USER'

    def test_drop_user_policy_incorrect(self):
        """
            Invoke drop_user() with policy incorrect
        """
        policy = {'timeout': 1000}
        user = "incorrect-policy"
        password = "foo1"
        roles = ["read", "read-write", "sys-admin"]

        status = self.client.admin_create_user( user, password, roles, policy )

        time.sleep(1)

        assert status == 0
        user_details = self.client.admin_query_user( user, policy )

        assert user_details == ['read', 'read-write', 'sys-admin']
        policy = {
            'timeout': 0.2
        }
        try:
            status = self.client.admin_drop_user( user, policy )

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == 'timeout is invalid'

        status = self.client.admin_drop_user( user )

    def test_drop_user_with_extra_argument(self):
        """
            Invoke drop_user() with extra argument.
        """
        policy = {'timeout': 1000}
        with pytest.raises(TypeError) as typeError:
            self.client.admin_drop_user( "foo-test", policy, "" )

        assert "admin_drop_user() takes at most 2 arguments (3 given)" in typeError.value

    def test_drop_user_with_too_long_username(self):

        policy = {}
        user = "user$" * 1000
        password = "user10"
        roles = ["sys-admin"]

        try:
            status = self.client.admin_create_user( user, password, roles, policy )

        except InvalidUser as exception:
            assert exception.code == 60
            assert exception.msg == "AEROSPIKE_INVALID_USER"

        try:
            status = self.client.admin_drop_user( user, policy )

        except InvalidUser as exception:
            assert exception.code == 60
            assert exception.msg == "AEROSPIKE_INVALID_USER"

    def test_drop_user_with_special_characters_in_username(self):

        policy = {}
        user = "!#Q#AEQ@#$%&^*((^&*~~~````"
        password = "user4"
        roles = ["read-write"]

        try:
            status = self.client.admin_create_user( user, password, roles, policy )
            assert status == 0
        except:
            pass

        status = self.client.admin_drop_user( user )

        assert status == 0



# -*- coding: utf-8 -*-

import pytest
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")


class SomeClass(object):

    pass


class TestGetKeyDigest(object):
    def setup_class(cls):
        """
        Setup method.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestGetKeyDigest.client = aerospike.client(config).connect()
        else:
            TestGetKeyDigest.client = aerospike.client(config).connect(
                user, password)

    def teardown_class(cls):
        TestGetKeyDigest.client.close()

    def setup_method(self, method):

        key = ('test', 'demo', "get_digest_key")
        rec = {'name': 'name1', 'age': 1}
        TestGetKeyDigest.client.put(key, rec)

    def teardown_method(self, method):
        """
        Teardoen method.
        """
        key = ('test', 'demo', "get_digest_key")
        TestGetKeyDigest.client.remove(key)

    def test_get_key_digest_with_no_parameter(self):
        """
            Invoke get_key_digest() without any mandatory parameters.
        """
        with pytest.raises(TypeError) as typeError:
            TestGetKeyDigest.client.get_key_digest()

        assert "Required argument 'ns' (pos 1) not found" in typeError.value

    def test_get_key_digest_with_only_ns(self):
        """
            Invoke get_key_digest() with a ns and no other parameters
        """
        with pytest.raises(TypeError) as typeError:
            TestGetKeyDigest.client.get_key_digest("test")

        assert "Required argument 'set' (pos 2) not found" in typeError.value

    def test_get_key_digest_with_integer_key(self):
        """
            Invoke get_key_digest() with integer key
        """

        digest = TestGetKeyDigest.client.get_key_digest("test", "demo", 1)

        assert type(digest) == bytearray

    def test_get_key_digest_with_string_key(self):
        """
            Invoke get_key_digest() with string key
        """

        digest = TestGetKeyDigest.client.get_key_digest("test", "demo",
                                                        "get_digest_key")

        assert type(digest) == bytearray

    def test_get_key_digest_with_integer_ns(self):
        """
            Invoke get_key_digest() with integer ns
        """

        with pytest.raises(TypeError) as typeError:
            digest = TestGetKeyDigest.client.get_key_digest(1, "demo",
                                                            "get_digest_key")

        assert 'Namespace should be a string' in typeError.value

    def test_get_key_digest_with_integer_set(self):
        """
            Invoke get_key_digest() with integer set
        """

        with pytest.raises(TypeError) as typeError:
            digest = TestGetKeyDigest.client.get_key_digest("test", 1,
                                                            "get_digest_key")

        assert 'Set should be a string or unicode' in typeError.value

    def test_get_key_digest_with_list_key(self):

        #Invoke get_key_digest() with key as a list
        mylist = [1, 2, 3]

        with pytest.raises(TypeError) as typeError:
            TestGetKeyDigest.client.get_key_digest("test", "demo", mylist)

        assert 'Key is invalid' in typeError.value

    def test_get_key_digest_with_map_key(self):
        #Invoke get_key_digest() with key as a map
        with pytest.raises(TypeError) as typeError:
            digest = TestGetKeyDigest.client.get_key_digest("test", "set",
                                                            {"a": 1,
                                                             "b": 2})

        assert 'Key is invalid' in typeError.value

    def test_get_key_digest_with_none_key(self):
        """
            Invoke get_key_digest() with key as a None
        """

        with pytest.raises(TypeError) as typeError:
            digest = TestGetKeyDigest.client.get_key_digest("test", "demo",
                                                            None)

        assert 'Key is invalid' in typeError.value



# -*- coding: utf-8 -*-
import pytest
import time
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestPrepend(object):
    def setup_class(cls):
        """
        Setup method.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestPrepend.client = aerospike.client(config).connect()
        else:
            TestPrepend.client = aerospike.client(config).connect(user,
                                                                  password)

    def teardown_class(cls):
        TestPrepend.client.close()

    def setup_method(self, method):
        for i in xrange(5):
            key = ('test', 'demo', i)
            rec = {'name': 'name%s' % (str(i)), 'age': i, 'nolist': [1, 2, 3]}
            TestPrepend.client.put(key, rec)
        key = ('test', 'demo', 'bytearray_key')
        TestPrepend.client.put(key, {"bytearray_bin": bytearray("asd;as[d'as;d",
            "utf-8")})

    def teardown_method(self, method):
        """
        Teardoen method.
        """
        for i in xrange(5):
            key = ('test', 'demo', i)
            TestPrepend.client.remove(key)
        key = ('test', 'demo', 'bytearray_key')
        TestPrepend.client.remove(key)

    def test_prepend_with_no_parameters(self):
        """
        Invoke prepend() without any mandatory parameters.
        """
        with pytest.raises(TypeError) as typeError:
            TestPrepend.client.prepend()
        assert "Required argument 'key' (pos 1) not found" in typeError.value

    def test_prepend_with_correct_paramters(self):
        """
        Invoke prepend() with correct parameters
        """
        key = ('test', 'demo', 1)
        TestPrepend.client.prepend(key, "name", "str")

        (key, meta, bins) = TestPrepend.client.get(key)

        assert bins == {'age': 1, 'name': 'strname1', 'nolist': [1, 2, 3]}

    def test_prepend_with_correct_policy(self):
        """
        Invoke prepend() with correct policy
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'commit_level': aerospike.POLICY_COMMIT_LEVEL_ALL
        }

        TestPrepend.client.prepend(key, "name", "str", {}, policy)

        (key, meta, bins) = TestPrepend.client.get(key)

        assert bins == {'age': 1, 'name': 'strname1', 'nolist': [1, 2, 3]}

    def test_prepend_with_policy_key_send(self):
        """
        Invoke prepend() with policy key send
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'commit_level': aerospike.POLICY_COMMIT_LEVEL_MASTER
        }
        TestPrepend.client.prepend(key, "name", "str", {}, policy)

        (key, meta, bins) = TestPrepend.client.get(key)

        assert bins == {'age': 1, 'name': 'strname1', 'nolist': [1, 2, 3]}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_prepend_with_policy_key_gen_EQ_ignore(self):
        """
        Invoke prepend() with gen eq positive
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'gen': aerospike.POLICY_GEN_IGNORE
        }

        meta = {'gen': 10, 'ttl': 1200}
        TestPrepend.client.prepend(key, "name", "str", meta, policy)

        (key, meta, bins) = TestPrepend.client.get(key)

        assert bins == {'age': 1, 'name': 'strname1', 'nolist': [1, 2, 3]}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_prepend_with_policy_key_gen_EQ_positive(self):
        """
        Invoke prepend() with gen eq positive
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'gen': aerospike.POLICY_GEN_EQ
        }
        (key, meta) = TestPrepend.client.exists(key)

        gen = meta['gen']
        meta = {'gen': gen, 'ttl': 1200}
        TestPrepend.client.prepend(key, "name", "str", meta, policy)

        (key, meta, bins) = TestPrepend.client.get(key)

        assert bins == {'age': 1, 'name': 'strname1', 'nolist': [1, 2, 3]}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_prepend_with_policy_key_gen_EQ_not_equal(self):
        """
        Invoke prepend() with policy key GEN_EQ not equal
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'gen': aerospike.POLICY_GEN_EQ
        }
        (key, meta) = TestPrepend.client.exists(key)
        gen = meta['gen']
        
        meta = {
            'gen': gen + 5,
            'ttl': 1200
        }
        try:
            TestPrepend.client.prepend(key, "name", "str", meta, policy)

        except RecordGenerationError as exception:
            assert exception.code == 3
            assert exception.msg == "AEROSPIKE_ERR_RECORD_GENERATION"
            assert exception.bin == 'name'

        (key, meta, bins) = TestPrepend.client.get(key)

        assert bins == {'age': 1, 'name': 'name1', 'nolist': [1, 2, 3]}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_prepend_with_policy_key_gen_GT_lesser(self):
        """
        Invoke prepend() with gen GT positive lesser
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'gen': aerospike.POLICY_GEN_GT
        }
        (key, meta) = TestPrepend.client.exists(key)

        gen = meta['gen']
        meta = {
            'gen': gen,
            'ttl': 1200
        }
        try:
            TestPrepend.client.prepend(key, "name", "str", meta, policy)

        except RecordGenerationError as exception:
            assert exception.code == 3
            assert exception.msg == "AEROSPIKE_ERR_RECORD_GENERATION"
            assert exception.bin == "name"

        (key, meta, bins) = TestPrepend.client.get(key)

        assert bins == {'age': 1, 'name': 'name1', 'nolist': [1, 2, 3]}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_prepend_with_policy_key_gen_GT_positive(self):
        """
        Invoke prepend() with gen GT positive
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'gen': aerospike.POLICY_GEN_GT
        }
        (key, meta) = TestPrepend.client.exists(key)

        gen = meta['gen']
        meta = {'gen': gen + 2, 'ttl': 1200}
        TestPrepend.client.prepend(key, "name", "str", meta, policy)

        (key, meta, bins) = TestPrepend.client.get(key)

        assert bins == {'age': 1, 'name': 'strname1', 'nolist': [1, 2, 3]}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_prepend_with_policy_key_digest(self):
        """
        Invoke prepend() with policy key digest
        """
        key = ('test', 'demo', None, bytearray("asd;as[d'as;djk;uyfl",
                                               "utf-8"))
        rec = {'name': 'name%s' % (str(1)), 'age': 1, 'nolist': [1, 2, 3]}
        TestPrepend.client.put(key, rec)

        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_DIGEST,
            'retry': aerospike.POLICY_RETRY_NONE
        }
        TestPrepend.client.prepend(key, "name", "str", {}, policy)

        (key, meta, bins) = TestPrepend.client.get(key)

        assert bins == {'age': 1, 'name': 'strname1', 'nolist': [1, 2, 3]}
        assert key == ('test', 'demo', None,
                       bytearray(b"asd;as[d\'as;djk;uyfl"))

        TestPrepend.client.remove(key)

    """
    def test_prepend_with_correct_policyandlist(self):
        #Invoke prepend() with correct policy
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key' : aerospike.POLICY_KEY_SEND
        }
        TestPrepend.client.prepend(key, "age", "str", policy)


        (key , meta, bins) = TestPrepend.client.get(key)

        assert bins == { 'age': 1, 'name': 'strname1', 'nolist': [1, 2, 3]}
    """

    def test_prepend_with_incorrect_policy(self):
        """
        Invoke prepend() with incorrect policy
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 0.5
        }
        try:
            TestPrepend.client.prepend(key, "name", "str", {}, policy)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "timeout is invalid"

    def test_prepend_with_nonexistent_key(self):
        """
        Invoke prepend() with non-existent key
        """
        key = ('test', 'demo', 1000)
        status = TestPrepend.client.prepend(key, "name", "str")

        assert status == 0L
        TestPrepend.client.remove(key)

    def test_prepend_with_nonexistent_bin(self):
        """
        Invoke prepend() with non-existent bin
        """
        key = ('test', 'demo', 1)
        status = TestPrepend.client.prepend(key, "name1", "str")

        assert status == 0L

    def test_prepend_value_not_string(self):
        """
        Invoke prepend() not a string
        """
        key = ('test', 'demo', 1)
        try:
            TestPrepend.client.prepend(key, "name", 2)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Cannot concatenate 'str' and 'non-str' objects"

    def test_prepend_with_extra_parameter(self):
        """
        Invoke prepend() with extra parameter.
        """
        key = ('test', 'demo', 1)
        policy = {'timeout': 1000}
        with pytest.raises(TypeError) as typeError:
            TestPrepend.client.prepend(key, "name", "str", {}, policy, "")

        assert "prepend() takes at most 5 arguments (6 given)" in typeError.value

    def test_prepend_policy_is_string(self):
        """
        Invoke prepend() with policy is string
        """
        key = ('test', 'demo', 1)
        try:
            TestPrepend.client.prepend(key, "name", "abc", {}, "")

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "policy must be a dict"

    def test_prepend_key_is_none(self):
        """
        Invoke prepend() with key is none
        """
        try:
            TestPrepend.client.prepend(None, "name", "str")

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "key is invalid"

    def test_prepend_bin_is_none(self):
        """
        Invoke prepend() with bin is none
        """
        key = ('test', 'demo', 1)
        try:
            TestPrepend.client.prepend(key, None, "str")

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Bin name should be of type string"
    
    def test_prepend_unicode_string(self):
        """
        Invoke prepend() with unicode string
        """
        key = ('test', 'demo', 1)
        res = TestPrepend.client.prepend(key, "name", u"age")

        key, meta, bins = TestPrepend.client.get(key)
        assert bins['name'] == 'agename1'

    def test_prepend_unicode_bin_name(self):
        """
        Invoke prepend() with unicode string
        """
        key = ('test', 'demo', 1)
        res = TestPrepend.client.prepend(key, u"add", u"address")

        key, meta, bins = TestPrepend.client.get(key)
        assert bins['add'] == 'address'

    def test_prepend_with_correct_parameters_without_connection(self):
        """
        Invoke prepend() with correct parameters without connection
        """
        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)
        key = ('test', 'demo', 1)

        try:
            client1.prepend(key, "name", "str")

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'

    def test_prepend_with_bytearray(self):
        """
        Invoke prepend() with bytearray value
        """
        key = ('test', 'demo', 'bytearray_key')
        TestPrepend.client.prepend(key, "bytearray_bin", bytearray("abc"))

        (key, meta, bins) = TestPrepend.client.get(key)

        assert bins == {'bytearray_bin': "abcasd;as[d'as;d"}

    def test_prepend_with_bytearray_new_key(self):
        """
        Invoke prepend() with bytearray value with a new record(non-existing)
        """
        key = ('test', 'demo', 'bytearray_new')
        TestPrepend.client.prepend(key, "bytearray_bin", bytearray("asd;as[d'as;d", "utf-8"))

        (key, meta, bins) = TestPrepend.client.get(key)

        assert bins == {'bytearray_bin': "asd;as[d'as;d"}

        TestPrepend.client.remove(key)



# -*- coding: utf-8 -*-
import pytest
import time
import sys
import cPickle as pickle
import socket
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestInfoNode(object):
    def setup_class(cls):
        """
        Setup class.
        """
        TestInfoNode.hostlist, user, password = TestBaseClass.get_hosts()
        config = {
                'hosts': TestInfoNode.hostlist
                }
        TestInfoNode.config = config
        if user == None and password == None:
            TestInfoNode.client = aerospike.client(config).connect()
        else:
            TestInfoNode.client = aerospike.client(config).connect(user,
                                                                   password)

    def teardown_class(cls):
        """
        Teardown class.
        """
        TestInfoNode.client.close()

    def test_info_node_no_parameters(self):
        """
        Test info with no parameters.
        """
        with pytest.raises(TypeError) as typeError:
            response = TestInfoNode.client.info_node()
        assert "Required argument 'command' (pos 1) not found" in typeError.value

    def test_info_node_positive(self):
        """
        Test info with correct arguments
        """
        key = ('test', 'demo', 'list_key')

        rec = {'names': ['John', 'Marlen', 'Steve']}

        TestInfoNode.client.put(key, rec)
        response = TestInfoNode.client.info_node('bins', TestInfoNode.config['hosts'][0])
        TestInfoNode.client.remove(key)
        if 'names' in response:
            assert True == True
        else:
            assert True == False

    def test_info_node_positive_for_namespace(self):
        """
        Test info with correct arguments
        """
        key = ('test', 'demo', 'list_key')

        rec = {'names': ['John', 'Marlen', 'Steve']}

        TestInfoNode.client.put(key, rec)
        response = TestInfoNode.client.info_node('namespaces', TestInfoNode.config['hosts'][0])
        TestInfoNode.client.remove(key)
        if 'test' in response:
            assert True == True
        else:
            assert True == False

    def test_info_node_positive_for_sets(self):
        """
        Test info with correct arguments
        """
        key = ('test', 'demo', 'list_key')

        rec = {'names': ['John', 'Marlen', 'Steve']}

        TestInfoNode.client.put(key, rec)
        response = TestInfoNode.client.info_node('sets', TestInfoNode.config['hosts'][0])
        TestInfoNode.client.remove(key)
        if 'demo' in response:
            assert True == True
        else:
            assert True == False

    def test_info_node_positive_for_sindex_creation(self):
        """
        Test info with correct arguments
        """
        try:
            TestInfoNode.client.index_remove('test','names_test_index')
            time.sleep(2)
        except:
            pass
        key = ('test', 'demo', 'list_key')

        rec = {'names': ['John', 'Marlen', 'Steve']}
        policy = {}
        TestInfoNode.client.put(key, rec)
        response = TestInfoNode.client.info_node('sindex-create:ns=test;set=demo;indexname=names_test_index;indexdata=names,string', TestInfoNode.config['hosts'][0])
        time.sleep(2)
        TestInfoNode.client.remove(key)
        response = TestInfoNode.client.info_node('sindex', TestInfoNode.config['hosts'][0])

        if 'names_test_index' in response:
            assert True == True
        else:
            assert True == False

    def test_info_node_for_incorrect_command(self):
        """
        Test info for incorrect command
        """
        response = None
        try:
            response = TestInfoNode.client.info_node('abcd', TestInfoNode.config['hosts'][0])

        except ClientError as exception:
            assert exception.code == -1
            assert exception.msg == "Invalid info operation"

    def test_info_node_positive_with_correct_policy(self):
        """
        Test info with correct policy
        """
        key = ('test', 'demo', 'list_key')

        rec = {'names': ['John', 'Marlen', 'Steve']}
        TestInfoNode.client.put(key, rec)

        host = ()
        policy = {'timeout': 1000}
        response = TestInfoNode.client.info_node('bins', host, policy)
        TestInfoNode.client.remove(key)
        if 'names' in response:
            assert True == True
        else:
            assert True == False

    def test_info_node_positive_with_incorrect_policy(self):
        """
        Test info with incorrect policy
        """
        host = ()
        policy = {
            'timeout': 0.5
        }
        try:
            response = TestInfoNode.client.info_node('bins', host, policy)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "timeout is invalid"

    def test_info_node_positive_with_host(self):
        """
        Test info with correct host
        """
        key = ('test', 'demo', 'list_key')

        rec = {'names': ['John', 'Marlen', 'Steve']}
        TestInfoNode.client.put(key, rec)
        host = TestInfoNode.config['hosts'][0]
        response = TestInfoNode.client.info_node('bins', host)

        TestInfoNode.client.remove(key)
        if 'names' in response:
            assert True == True
        else:
            assert True == False

    def test_info_node_positive_with_incorrect_host(self):
        """
        Test info with incorrect host
        """
        host = ("122.0.0.1", 3000)
        try:
            response = TestInfoNode.client.info_node('bins', host)

        except TimeoutError as exception:
            assert exception.code == 9L
            assert exception.msg == ""

    def test_info_node_positive_with_all_parameters(self):
        """
        Test info with all parameters
        """
        policy = {
            'timeout': 1000
        }
        host = TestInfoNode.config['hosts'][0]
        response = TestInfoNode.client.info_node('logs', host, policy)

        assert response != None

    def test_info_node_positive_with_extra_parameters(self):
        """
        Test info with extra parameters
        """
        host = TestInfoNode.config['hosts'][0]
        policy = {'timeout': 1000}
        with pytest.raises(TypeError) as typeError:
            response = TestInfoNode.client.info_node('bins', host, policy, "")

        assert "info_node() takes at most 3 arguments (4 given)" in typeError.value

    def test_info_node_for_none_command(self):
        """
        Test info for None command
        """
        response = None
        try:
            response = TestInfoNode.client.info_node(None, TestInfoNode.config['hosts'][0])

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Request should be of string type"

    def test_info_node_with_unicode_request_string_and_host_name(self):
        """
        Test info with all parameters
        """
        host = (unicode(TestInfoNode.config['hosts'][0][0]),
                TestInfoNode.config['hosts'][0][1])
        policy = {
            'timeout': 1000
        }
        response = TestInfoNode.client.info_node(u'logs', host, policy)

    def test_info_node_positive_with_unicode_request_string_and_host_name(self
                                                                         ):
        """
        Test info with all parameters
        """
        host = (unicode(TestInfoNode.config['hosts'][0][0]),
                TestInfoNode.config['hosts'][0][1])
        policy = {'timeout': 1000}
        response = TestInfoNode.client.info_node(u'logs', host, policy)

        assert response != None

    def test_info_node_positive_with_valid_host(self):
        """
        Test info with incorrect host
        """
        host = ( "192.168.244.244", 3000 )
        try:
            response = TestInfoNode.client.info_node('bins', host)
        except ClientError as exception:
            assert exception.code == -1L
        except TimeoutError as exception:
            assert exception.code == 9L
    
    def test_info_node_positive_invalid_host(self):
        """
        Test info with incorrect host
        """
        host = ("abcderp", 3000)
        with pytest.raises(Exception) as exception:
            response = TestInfoNode.client.info_node('bins', host)

        assert exception.value[0] == -4L

    def test_info_node_positive_with_dns(self):
        """
        Test info with incorrect host
        """
        host = ( "google.com", 3000 )
        try:
            response = TestInfoNode.client.info_node('bins', host)

        except TimeoutError as exception:
            assert exception.code == 9L
            assert exception.msg == ""

    def test_info_node_positive_without_connection(self):
        """
        Test info with correct arguments without connection
        """
        key = ('test', 'demo', 'list_key')

        client1 = aerospike.client(TestInfoNode.config)
        try:
            response = client1.info_node('bins', TestInfoNode.config['hosts'][0])

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'



# -*- coding: utf-8 -*-
import pytest
import time
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    import aerospike
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestTouch(object):
    def setup_class(cls):
        """
        Setup method.
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestTouch.client = aerospike.client(config).connect()
        else:
            TestTouch.client = aerospike.client(config).connect(user, password)

    def teardown_class(cls):
        TestTouch.client.close()

    def setup_method(self, method):
        for i in xrange(5):
            key = ('test', 'demo', i)
            rec = {'name': 'name%s' % (str(i)), 'age': i}
            TestTouch.client.put(key, rec)

    def teardown_method(self, method):
        """
        Teardoen method.
        """
        for i in xrange(5):
            key = ('test', 'demo', i)
            TestTouch.client.remove(key)

    def test_touch_with_no_parameters(self):
        """
        Invoke touch() without any mandatory parameters.
        """
        with pytest.raises(TypeError) as typeError:
            TestTouch.client.touch()
        assert "Required argument 'key' (pos 1) not found" in typeError.value

    def test_touch_with_correct_paramters(self):
        """
        Invoke touch() with correct parameters
        """
        key = ('test', 'demo', 1)
        response = TestTouch.client.touch(key, 120)

        assert response == 0

    def test_touch_with_correct_policy(self):
        """
        Invoke touch() with correct policy
        """
        key = ('test', 'demo', 1)
        policy = {'timeout': 1000, 'retry': aerospike.POLICY_RETRY_ONCE}
        response = TestTouch.client.touch(key, 120, {}, policy)
        assert response == 0

    def test_touch_with_policy_key_send(self):
        """
        Invoke touch() with policy key send
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'commit_level': aerospike.POLICY_COMMIT_LEVEL_MASTER
        }
        TestTouch.client.touch(key, 120, {}, policy)

        (key, meta, bins) = TestTouch.client.get(key)

        assert bins == {'age': 1, 'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_touch_with_policy_key_digest(self):
        """
        Invoke touch() with policy key digest
        """
        key = ('test', 'demo', None, bytearray("asd;as[d'as;djk;uyfl",
                                               "utf-8"))
        rec = {'name': 'name%s' % (str(1)), 'age': 1, 'nolist': [1, 2, 3]}
        TestTouch.client.put(key, rec)

        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_DIGEST,
            'retry': aerospike.POLICY_RETRY_NONE
        }
        TestTouch.client.touch(key, 120, {}, policy)

        (key, meta, bins) = TestTouch.client.get(key)

        assert bins == {'age': 1, 'name': 'name1', 'nolist': [1, 2, 3]}
        assert key == ('test', 'demo', None,
                       bytearray(b"asd;as[d\'as;djk;uyfl"))
        TestTouch.client.remove(key)

    def test_touch_with_policy_key_gen_EQ_ignore(self):
        """
        Invoke touch() with gen eq positive ignore
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'gen': aerospike.POLICY_GEN_IGNORE
        }

        meta = {'gen': 10, 'ttl': 1200}
        TestTouch.client.touch(key, 120, meta, policy)

        (key, meta, bins) = TestTouch.client.get(key)

        assert bins == {'age': 1, 'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_touch_with_policy_key_gen_EQ_positive(self):
        """
        Invoke touch() with gen eq positive
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'gen': aerospike.POLICY_GEN_EQ
        }
        (key, meta) = TestTouch.client.exists(key)

        gen = meta['gen']
        meta = {'gen': gen, 'ttl': 1200}
        TestTouch.client.touch(key, 120, meta, policy)

        (key, meta, bins) = TestTouch.client.get(key)

        assert bins == {'age': 1, 'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_touch_with_policy_key_gen_EQ_not_equal(self):
        """
        Invoke touch() with policy key EQ not equal
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'gen': aerospike.POLICY_GEN_EQ
        }
        meta = {
            'gen': 10,
            'ttl': 1200
        }
        try:
            TestTouch.client.touch(key, 120, meta, policy)

        except RecordGenerationError as exception:
            assert exception.code == 3
            assert exception.msg == "AEROSPIKE_ERR_RECORD_GENERATION"

        (key, meta, bins) = TestTouch.client.get(key)

        assert bins == {'age': 1, 'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_touch_with_policy_key_gen_GT_lesser(self):
        """
        Invoke touch() with gen GT lesser
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'gen': aerospike.POLICY_GEN_GT
        }
        (key, meta) = TestTouch.client.exists(key)

        gen = meta['gen']
        meta = {
            'gen': gen,
            'ttl': 1200
        }
        try:
            TestTouch.client.touch(key, 120, meta, policy)

        except RecordGenerationError as exception:
            assert exception.code == 3
            assert exception.msg == "AEROSPIKE_ERR_RECORD_GENERATION"

        (key, meta, bins) = TestTouch.client.get(key)

        assert bins == {'age': 1, 'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_touch_with_policy_key_gen_GT_positive(self):
        """
        Invoke touch() with gen GT positive
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'key': aerospike.POLICY_KEY_SEND,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'gen': aerospike.POLICY_GEN_GT
        }
        (key, meta) = TestTouch.client.exists(key)

        gen = meta['gen']
        meta = {'gen': gen + 5, 'ttl': 1200}
        TestTouch.client.touch(key, 120, meta, policy)

        (key, meta, bins) = TestTouch.client.get(key)

        assert bins == {'age': 1, 'name': 'name1'}
        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )

    def test_touch_with_incorrect_policy(self):
        """
        Invoke touch() with incorrect policy
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 0.5
        }
        try:
            TestTouch.client.touch(key, 120, {}, policy)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "timeout is invalid"

    def test_touch_with_nonexistent_key(self):
        """
        Invoke touch() with non-existent key
        """
        key = ('test', 'demo', 1000)

        try:
            status = TestTouch.client.touch(key, 120)

        except RecordNotFound as exception:
            assert exception.code == 2
            assert exception.msg == "AEROSPIKE_ERR_RECORD_NOT_FOUND"

    def test_touch_value_string(self):
        """
        Invoke touch() not a string
        """
        key = ('test', 'demo', 1)
        try:
            TestTouch.client.touch(key, "name")

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Unsupported operand type(s) for touch : only int or long allowed"

    def test_touch_with_extra_parameter(self):
        """
        Invoke touch() with extra parameter.
        """
        key = ('test', 'demo', 1)
        policy = {'timeout': 1000}
        with pytest.raises(TypeError) as typeError:
            TestTouch.client.touch(key, 120, {}, policy, "")

        assert "touch() takes at most 4 arguments (5 given)" in typeError.value

    def test_touch_policy_is_string(self):
        """
        Invoke touch() with policy is string
        """
        key = ('test', 'demo', 1)
        try:
            TestTouch.client.touch(key, 120, {}, "")

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "policy must be a dict"

    def test_touch_with_correct_paramters_without_connection(self):
        """
        Invoke touch() with correct parameters without connection
        """
        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)
        key = ('test', 'demo', 1)

        try:
            response = client1.touch(key, 120)

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'

    def test_touch_withttlvalue_greaterthan_maxsize(self):
        """
        Invoke touch() with ttl value greater than (2^63-1)
        """
        key = ('test', 'demo', 1)
        meta = {'gen': 10, 'ttl': 12005678901234567890}
        try:
            TestTouch.client.touch(key, 120, meta, None)
        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'integer value for ttl exceeds sys.maxsize'



# -*- coding: utf-8 -*-

import pytest
import sys
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestRemove(TestBaseClass):
    def setup_class(cls):
        """
        Setup class
        """
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        if user == None and password == None:
            TestRemove.client = aerospike.client(config).connect()
        else:
            TestRemove.client = aerospike.client(config).connect(user, password)

    def teardown_class(cls):
        TestRemove.client.close()

    def setup_method(self, method):
        """
        Setup method.
        """
        for i in xrange(5):
            key = ('test', 'demo', i)
            rec = {
                'name': 'name%s' % (str(i)),
                'addr': 'name%s' % (str(i)),
                'age': i,
                'no': i
            }
            TestRemove.client.put(key, rec)

    def teardown_method(self, method):
        """
        Teardoen method.
        """
        for i in xrange(5):
            try:
                key = ('test', 'demo', i)
                TestRemove.client.remove(key)
            except RecordError:
                pass

    def test_remove_with_no_parameters(self):
        """
            Invoke remove() without any mandatory parameters.
        """
        with pytest.raises(TypeError) as typeError:
            TestRemove.client.remove()

        assert "Required argument 'key' (pos 1) not found" in typeError.value

    def test_remove_with_correct_parameters(self):
        """
            Invoke remove() with correct arguments
        """
        key = ('test', 'demo', 1)
        retobj = TestRemove.client.remove(key)

        assert retobj == 0L

        try:
            (key, meta, bins) = TestRemove.client.get(key)

        except RecordNotFound as exception:
            assert exception.code == 2
        
        key = ('test', 'demo', 1)
        rec = {
            'name': 'name%s' % (str(1)),
            'addr': 'name%s' % (str(1)),
            'age': 1,
            'no': 1
        }
        TestRemove.client.put(key, rec)

    def test_remove_with_policy(self):
        """
            Invoke remove() with policy
        """
        key = ('test', 'demo', 1)
        meta = {'gen': 0}
        policy = {'timeout': 1000}

        retobj = TestRemove.client.remove(key, meta, policy)

        assert retobj == 0L

        try:
            (key, meta, bins) = TestRemove.client.get(key)

        except RecordNotFound as exception:
            assert exception.code == 2
    
        key = ('test', 'demo', 1)
        rec = {
            'name': 'name%s' % (str(1)),
            'addr': 'name%s' % (str(1)),
            'age': 1,
            'no': 1
        }
        TestRemove.client.put(key, rec)

    def test_remove_with_policy_all(self):
        """
            Invoke remove() with policy
        """
        key = ('test', 'demo', 1)
        meta = {'gen': 0}
        policy = {
            'timeout': 1000,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND
        }
        retobj = TestRemove.client.remove(key, meta, policy)

        assert retobj == 0L

        try:
            (key, meta, bins) = TestRemove.client.get(key)

        except RecordNotFound as exception:
            assert exception.code == 2

        key = ('test', 'demo', 1)
        rec = {
            'name': 'name%s' % (str(1)),
            'addr': 'name%s' % (str(1)),
            'age': 1,
            'no': 1
        }
        TestRemove.client.put(key, rec)

    def test_remove_with_policy_key_digest(self):
        """
            Invoke remove() with policy_key_digest
        """

        key = ('test', 'demo', None, bytearray("asd;as[d'as;djk;uyfl",
                                               "utf-8"))
        meta = {'gen': 0}
        policy = {
            'timeout': 1000,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_DIGEST
        }
        retobj = TestRemove.client.put(key, policy)

        assert retobj == 0L

        retobj = TestRemove.client.remove(key, meta, policy)

        assert retobj == 0L

        try:
            (key, meta, bins) = TestRemove.client.get(key)

        except RecordNotFound as exception:
            assert exception.code == 2
        
    def test_remove_with_policy_gen_ignore(self):
        """
            Invoke remove() with policy gen ignore
        """
        key = ('test', 'demo', 1)
        meta = {'gen': 0}
        policy = {
            'timeout': 1000,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND,
            'gen': aerospike.POLICY_GEN_IGNORE
        }

        retobj = TestRemove.client.remove(key, meta, policy)

        assert retobj == 0L

        try:
            (key, meta, bins) = TestRemove.client.get(key)

        except RecordNotFound as exception:
            assert exception.code == 2

        key = ('test', 'demo', 1)
        rec = {
            'name': 'name%s' % (str(1)),
            'addr': 'name%s' % (str(1)),
            'age': 1,
            'no': 1
        }
        TestRemove.client.put(key, rec)

    def test_remove_with_policy_gen_eq_positive(self):
        """
            Invoke remove() with policy gen positive
        """
        key = ('test', 'demo', 1)
        meta = {'gen': 0}
        policy = {
            'timeout': 1000,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND,
            'gen': aerospike.POLICY_GEN_EQ
        }

        (key, meta) = TestRemove.client.exists(key)
        gen = meta['gen']
        retobj = TestRemove.client.remove(key, meta, policy)

        assert retobj == 0L

        try:
            (key, meta, bins) = TestRemove.client.get(key)

        except RecordNotFound as exception:
            assert exception.code == 2

        key = ('test', 'demo', 1)
        rec = {
            'name': 'name%s' % (str(1)),
            'addr': 'name%s' % (str(1)),
            'age': 1,
            'no': 1
        }
        TestRemove.client.put(key, rec)

    def test_remove_with_policy_gen_eq_not_equal(self):
        """
            Invoke remove() with policy gen not equal
        """
        key = ('test', 'demo', 1)

        (key, meta) = TestRemove.client.exists(key)
        gen = meta['gen'] + 5
        metadata = {'gen': gen}
        policy = {
            'timeout': 1000,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND,
            'gen': aerospike.POLICY_GEN_EQ
        }

        (key, meta) = TestRemove.client.exists(key)
        gen = meta['gen'] + 5

        try:
            retobj = TestRemove.client.remove(key, gen, policy)

        except RecordGenerationError as exception:
            assert exception.code == 3
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_GENERATION'

        (key, meta, bins) = TestRemove.client.get(key)

        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )
        assert meta != None
        assert bins == {'addr': 'name1', 'age': 1, 'name': 'name1', 'no': 1}

    def test_remove_with_policy_gen_GT_lesser(self):
        """
            Invoke remove() with policy gen GT lesser
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND,
            'gen': aerospike.POLICY_GEN_GT
        }

        (key, meta) = TestRemove.client.exists(key)
        gen = meta['gen']
        try:
            retobj = TestRemove.client.remove(key, gen, policy)

        except RecordGenerationError as exception:
            assert exception.code == 3
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_GENERATION'

        (key, meta, bins) = TestRemove.client.get(key)

        assert key == ('test', 'demo', None, bytearray(
            b'\xb7\xf4\xb88\x89\xe2\xdag\xdeh>\x1d\xf6\x91\x9a\x1e\xac\xc4F\xc8')
                      )
        assert meta != None
        assert bins == {'addr': 'name1', 'age': 1, 'name': 'name1', 'no': 1}

    def test_remove_with_policy_gen_GT_positive(self):
        """
            Invoke remove() with policy gen GT positive
        """
        key = ('test', 'demo', 1)
        policy = {
            'timeout': 1000,
            'retry': aerospike.POLICY_RETRY_ONCE,
            'key': aerospike.POLICY_KEY_SEND,
            'gen': aerospike.POLICY_GEN_GT
        }

        (key, meta) = TestRemove.client.exists(key)
        gen = meta['gen'] + 5
        metadata = {'gen': gen}
        retobj = TestRemove.client.remove(key, metadata, policy)

        assert retobj == 0L

        try:
            (key, meta, bins) = TestRemove.client.get(key)

        except RecordNotFound as exception:
            assert exception.code == 2

        key = ('test', 'demo', 1)
        rec = {
            'name': 'name%s' % (str(1)),
            'addr': 'name%s' % (str(1)),
            'age': 1,
            'no': 1
        }
        TestRemove.client.put(key, rec)

    def test_remove_with_policy_as_string(self):
        """
            Invoke remove() with policy as string
        """
        key = ('test', 'demo', 1)
        meta = {
                'gen' : 0
                }
        try:
            retobj = TestRemove.client.remove(key, meta, "")

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'policy must be a dict'

    def test_remove_with_extra_parameter(self):
        """
            Invoke remove() with extra parameter
        """
        key = ('test', 'demo', 1)
        meta = {'gen': 0}
        policy = {'timeout': 1000}
        with pytest.raises(TypeError) as typeError:
            retobj = TestRemove.client.remove(key, meta, policy, "")

        assert "remove() takes at most 3 arguments (4 given)" in typeError.value

    def test_remove_with_key_none(self):
        """
            Invoke remove() with key as None
        """
        try:
            retobj = TestRemove.client.remove(None)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'key is invalid'

    def test_remove_with_key_incorrect(self):
        """
            Invoke remove() with key incorrect
        """
        key = ('test', 'demo', 15)
        try:
            retobj = TestRemove.client.remove(key)

        except RecordNotFound as exception:
            assert exception.code == 2
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_NOT_FOUND'

    def test_remove_with_namespace_none(self):
        """
            Invoke remove() with namespace as None
        """
        key = (None, 'demo', 1)
        try:
            retobj = TestRemove.client.remove(key)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == 'namespace must be a string'

    def test_remove_with_set_none(self):
        """
            Invoke remove() with set as None
        """
        key = ('test', None, 1)
        try:
            retobj = TestRemove.client.remove(key)

        except RecordNotFound as exception:
            assert exception.code == 2
            assert exception.msg == 'AEROSPIKE_ERR_RECORD_NOT_FOUND'

    def test_remove_with_correct_parameters_without_connection(self):
        """
            Invoke remove() with correct arguments without connection
        """
        config = {'hosts': [('127.0.0.1', 3000)]}
        client1 = aerospike.client(config)
        key = ('test', 'demo', 1)

        try:
            retobj = client1.remove(key)

        except ClusterError as exception:
            assert exception.code == 11L
            assert exception.msg == 'No connection to aerospike cluster'



# -*- coding: utf-8 -*-

import pytest
import sys
import json
import cPickle as pickle
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestConnect(TestBaseClass):
    def setup_class(cls):
        hostlist, user, password = TestBaseClass.get_hosts()
        config = {'hosts': hostlist}
        TestConnect.config = config

    def test_version(self):
        """
            Check for aerospike vrsion
        """
        assert aerospike.__version__ != None

    def test_connect_positive(self):
        """
            Invoke connect() with positive parameters.
        """
        config = {'hosts': TestConnect.hostlist}
        if TestConnect.user == None and TestConnect.password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(
                TestConnect.user, TestConnect.password)

        assert self.client != None
        assert self.client.is_connected() == True
        self.client.close()

    def test_connect_positive_with_policy(self):
        """
            Invoke connect() with positive parameters and policy.
        """
        config = {
            'hosts': TestConnect.hostlist,
            'policies': {'timeout': 10000}
        }
        if TestConnect.user == None and TestConnect.password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(
                TestConnect.user, TestConnect.password)

        assert self.client.is_connected() == True
        assert self.client.is_connected() == True
        self.client.close()

    def test_connect_positive_with_multiple_hosts(self):
        """
            Invoke connect() with multiple hosts.
        """
        config = {'hosts': TestConnect.hostlist}
        if TestConnect.user == None and TestConnect.password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(
                TestConnect.user, TestConnect.password)

        assert self.client != None
        self.client.close()

    def test_connect_positive_unicode_hosts(self):
        """
            Invoke connect() with unicode hosts.
        """
        uni = json.dumps(TestConnect.hostlist[0])
        hostlist = json.loads(uni)
        config = {'hosts': [(hostlist[0], hostlist[1])],
                'policies': {'use_batch_direct': True}
                }
        if TestConnect.user == None and TestConnect.password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(
                TestConnect.user, TestConnect.password)

        assert self.client != None
        assert self.client.is_connected() == True
        self.client.close()

    def test_connect_config_not_dict(self):
        """
            Invoke connect with config as Integer (non-dict)
        """
        config = 1
        try:
            self.client = aerospike.client(config).connect()
        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Parameters are incorrect"
    
    def test_connect_config_empty_dict(self):
        """
            Invoke connect() with config as empty dict.
        """
        config = {
                }
        try:
            self.client = aerospike.client(config).connect()

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == 'No hosts provided'

    def test_connect_missing_hosts_key(self):
        """
            Invoke connect() with host key missing in config dict.
        """
        config = {
                '': [('127.0.0.1', 3000)]
                }
        try:
            self.client = aerospike.client(config).connect()

        except ParamError as exception:
            assert exception.code == -2L
            assert exception.msg == 'No hosts provided'

    def test_connect_missing_address(self):
        """
            Invoke connect() with missing address in config dict.
        """
        config = {
                'hosts': [3000]
                }
        try:
            self.client = aerospike.client(config).connect()

        except ParamError as exception:
            assert exception.code == -2L

    def test_connect_missing_port(self):
        """
            Invoke connect() with missing port in config dict.
        """
        config = {
                'hosts': [('127.0.0.1')]
                }
        self.client = aerospike.client(config)
        try:
            self.client.connect()
            self.client.close()
        except ClientError as exception:
            assert exception.code == -1
            assert self.client.is_connected() != True

    def test_connect_incorrect_port(self):
        """
            Invoke connect() with incorrect port in config dict.
        """
        config = {
                'hosts': [('127.0.0.1', 2000)]
                }
        try:
            self.client = aerospike.client(config).connect()

        except ClientError as exception:
            assert exception.code == -1
            assert exception.msg == 'Failed to seed cluster'

    def test_connect_port_is_string(self):
        """
            Invoke connect() with port as string in config dict.
        """
        config = {
                'hosts': [('127.0.0.1', '3000')]
                }
        try:
            self.client = aerospike.client(config).connect()

        except ClientError as exception:
            assert exception.code == -1
            assert exception.msg == 'Failed to seed cluster'

    def test_connect_positive_shm_key(self):
        """
            Invoke connect() with shm_key specified
        """
        config = {'hosts': TestConnect.hostlist,
                'shm': {'shm_key': 3}
                }
        if TestConnect.user == None and TestConnect.password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(
                TestConnect.user, TestConnect.password)

        assert self.client != None
        assert self.client.is_connected() == True
        assert self.client.shm_key() == 3
        self.client.close()

    def test_connect_positive_shm_key_default(self):
        """
            Invoke connect() with shm enabled but shm_key not specified
        """
        config = {'hosts': TestConnect.hostlist,
                'shm': {'shm_max_nodes': 5}
                }
        if TestConnect.user == None and TestConnect.password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(
                TestConnect.user, TestConnect.password)

        assert self.client != None
        assert self.client.is_connected() == True
        assert self.client.shm_key() == 2768240640
        self.client.close()

    def test_connect_positive_shm_not_enabled(self):
        """
            Invoke connect() with shm not anabled
        """
        config = {'hosts': TestConnect.hostlist,
                }
        if TestConnect.user == None and TestConnect.password == None:
            self.client = aerospike.client(config).connect()
        else:
            self.client = aerospike.client(config).connect(
                TestConnect.user, TestConnect.password)

        assert self.client != None
        assert self.client.is_connected() == True
        assert self.client.shm_key() == None
        self.client.close()



# -*- coding: utf-8 -*-

import pytest
import sys
import time
from test_base_class import TestBaseClass

aerospike = pytest.importorskip("aerospike")
try:
    from aerospike.exception import *
except:
    print "Please install aerospike python client."
    sys.exit(1)

class TestCreateRole(TestBaseClass):

    pytestmark = pytest.mark.skipif(
        TestBaseClass().get_hosts()[1] == None,
        reason="No user specified, may be not secured cluster.")

    def setup_method(self, method):

        """
        Setup method
        """
        hostlist, user, password = TestBaseClass().get_hosts()
        config = { "hosts": hostlist}
        self.client = aerospike.client(config).connect( user, password )
        try:
            self.client.admin_drop_user("testcreaterole")
        except:
            pass # do nothing, EAFP

        self.delete_users = []

    def teardown_method(self, method):

        """
        Teardown method
        """

        policy = {}

        for user in self.delete_users:
            self.client.admin_drop_user( user, policy )

        self.client.close()

    def test_create_role_without_any_parameters(self):

        with pytest.raises(TypeError) as typeError:
            self.client.admin_create_role()

        assert "Required argument 'role' (pos 1) not found" in typeError.value

    def test_create_role_positive_with_policy(self):
        """
            Create role positive
        """
        try:
            self.client.admin_query_role("usr-sys-admin-test")
            # role exists, clear it out.
            self.client.admin_drop_role("usr-sys-admin-test")
        except InvalidRole:
            pass # we are good, no such role exists

        self.client.admin_create_role("usr-sys-admin-test",
                [{"code": aerospike.PRIV_READ, "ns": "test", "set":"demo"}],
                {'timeout': 1000})
        time.sleep(1)
        roles = self.client.admin_query_role("usr-sys-admin-test")
        assert roles == [{'code': 10, 'ns': 'test', 'set': 'demo'}]

        status = self.client.admin_create_user("testcreaterole", "createrole", ["usr-sys-admin-test"])

        assert status == 0
        time.sleep(1)
        users = self.client.admin_query_user("testcreaterole")

        assert users == ["usr-sys-admin-test"]

        status = self.client.admin_drop_role("usr-sys-admin-test")

        assert status == 0

        users = self.client.admin_query_user("testcreaterole")

        assert users == []

        self.client.admin_drop_user("testcreaterole")

    def test_create_role_positive(self):
        """
            Create role positive
        """
        try:
            self.client.admin_query_role("usr-sys-admin-test")
            # role exists, clear it out.
            self.client.admin_drop_role("usr-sys-admin-test")
        except InvalidRole:
            pass # we are good, no such role exists

        self.client.admin_create_role("usr-sys-admin-test", [{"code":
            aerospike.PRIV_USER_ADMIN}, {"code": aerospike.PRIV_SYS_ADMIN}])
        time.sleep(1)
        roles = self.client.admin_query_role("usr-sys-admin-test")

        assert roles == [{"code": 0, 'ns': '', 'set': ''}, {"code": 1, 'ns': '', 'set': ''}]

        status = self.client.admin_create_user("testcreaterole", "createrole",
["usr-sys-admin-test"])

        assert status == 0
        time.sleep(1)
        users = self.client.admin_query_user("testcreaterole")

        assert users == ["usr-sys-admin-test"]

        status = self.client.admin_drop_role("usr-sys-admin-test")

        assert status == 0

        users = self.client.admin_query_user("testcreaterole")

        assert users == []

        self.client.admin_drop_user("testcreaterole")

    def test_create_role_incorrect_role_type(self):
        """
            role name not string
        """
        try:
            self.client.admin_create_role(1, [{"code": aerospike.PRIV_USER_ADMIN}])
        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Role name should be a string"

    def test_create_role_unknown_privilege_type(self):
        """
            privilege type unknown
        """
        try:
            self.client.admin_query_role("usr-sys-admin-test")
            # role exists, clear it out.
            self.client.admin_drop_role("usr-sys-admin-test")
        except InvalidRole:
            pass # we are good, no such role exists

        try:
            self.client.admin_create_role("usr-sys-admin-test", [{"code": 64}])
        except InvalidPrivilege as exception:
            assert exception.code == 72

    def test_create_role_incorrect_privilege_type(self):
        """
            privilege type incorrect
        """
        try:
            self.client.admin_create_role("usr-sys-admin-test", None)

        except ParamError as exception:
            assert exception.code == -2
            assert exception.msg == "Privileges should be a list"

    def test_create_role_existing_role(self):
        """
            create an already existing role
        """
        try:
            self.client.admin_query_role("usr-sys-admin-test")
            # role exists, clear it out.
            self.client.admin_drop_role("usr-sys-admin-test")
        except InvalidRole:
            pass # we are good, no such role exists

        self.client.admin_create_role("usr-sys-admin-test", [{"code":
            aerospike.PRIV_USER_ADMIN}, {"code": aerospike.PRIV_SYS_ADMIN}])
        try:
            self.client.admin_create_role("usr-sys-admin-test", [{"code":
                aerospike.PRIV_USER_ADMIN}, {"code": aerospike.PRIV_SYS_ADMIN}])

        except RoleExistsError as exception:
            assert exception.code == 71
            assert exception.msg == "AEROSPIKE_ROLE_ALREADY_EXISTS"

        time.sleep(1)
        status = self.client.admin_drop_role("usr-sys-admin-test")

        assert status == 0

    def test_create_role_positive_with_special_characters(self):
        """
            Create role positive with special characters in role name
        """
        role_name = "!#Q#AEQ@#$%&^*((^&*~~~````"
        try:
            self.client.admin_drop_role(role_name) # clear out if it exists
        except:
            pass # EAFP
        status = self.client.admin_create_role(role_name, [{"code": aerospike.PRIV_READ, "ns": "test", "set":"demo"}], {'timeout': 1000})

        assert status == 0
        time.sleep(1)
        roles = self.client.admin_query_role(role_name)

        assert roles == [{"code": aerospike.PRIV_READ, "ns": "test", "set": "demo"}]

        status = self.client.admin_create_user("testcreaterole", "createrole", [role_name])

        assert status == 0
        time.sleep(1)
        users = self.client.admin_query_user("testcreaterole")

        assert users == [role_name]

        status = self.client.admin_drop_role(role_name)

        assert status == 0

        users = self.client.admin_query_user("testcreaterole")

        assert users == []

        self.client.admin_drop_user("testcreaterole")

    def test_create_role_positive_with_too_long_role_name(self):
        """
            Create role positive with too long role name
        """
        role_name = "role$"*1000

        try:
            self.client.admin_create_role(role_name, [{"code":
aerospike.PRIV_READ, "ns": "test", "set":"demo"}], {'timeout': 1000})

        except InvalidRole as exception:
            assert exception.code == 70
            assert exception.msg == "AEROSPIKE_INVALID_ROLE"

